@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -37,6 +37,9 @@ class Balloon(object):
         # create inputbox
         self.inputbox = InputBox()
         self.inputbox.set_responsible(self.handle_request)
+        # create passwordinputbox
+        self.passwordinputbox = PasswordInputBox()
+        self.passwordinputbox.set_responsible(self.handle_request)
 
     def set_responsible(self, request_method):
         self.request_parent = request_method
@@ -80,10 +83,7 @@ class Balloon(object):
 
     def create_gtk_window(self, title):
         window = ninix.pix.TransparentWindow()
-        window.set_focus_on_map(False)
         window.set_title(title)
-        window.set_decorated(False)
-        window.set_resizable(False)
         window.set_skip_pager_hint(False)
         window.set_skip_taskbar_hint(True)
         window.connect('delete_event', self.delete)
@@ -106,6 +106,7 @@ class Balloon(object):
         self.communicatebox.destroy()
         self.teachbox.destroy()
         self.inputbox.destroy()
+        self.passwordinputbox.destroy()
 
     def new(self, desc, balloon):
         self.desc = desc
@@ -148,6 +149,8 @@ class Balloon(object):
         self.teachbox.new(desc, communicate2)
         # configure inputbox
         self.inputbox.new(desc, communicate3)
+        # configure passwordinputbox
+        self.passwordinputbox.new(desc, communicate3)
 
     def add_window(self, side):
         assert len(self.window) == side
@@ -190,17 +193,18 @@ class Balloon(object):
             return (0, 0)
 
     def set_balloon_default(self):
+        default_id = self.request_parent('GET', 'get_balloon_default_id')
+        try:
+            default_id = int(default_id)
+        except:
+            default_id = 0
         for side in range(len(self.window)):
-            self.window[side].set_balloon(0) ## FIXME
+            self.window[side].set_balloon(default_id)
 
     def set_balloon(self, side, num):
         if len(self.window) > side:
             self.window[side].set_balloon(num)
 
-    def set_direction(self, side, dir):
-        if len(self.window) > side:
-            self.window[side].set_direction(dir)
-
     def set_position(self, side, base_x, base_y):
         if len(self.window) > side:
             self.window[side].set_position(base_x, base_y)
@@ -211,6 +215,10 @@ class Balloon(object):
         else:
             return (0, 0)
 
+    def set_autoscroll(self, flag):
+        for side in range(len(self.window)):
+            self.window[side].set_autoscroll(flag)
+
     def is_shown(self, side):
         return 0 if len(self.window) <= side else self.window[side].is_shown()
 
@@ -343,6 +351,19 @@ class Balloon(object):
             self.inputbox.set_limittime(limittime)
             self.inputbox.show(default)
 
+    def open_passwordinputbox(self, symbol, limittime=-1, default=None):
+        if not self.user_interaction:
+            self.user_interaction = True
+            self.passwordinputbox.set_symbol(symbol)
+            self.passwordinputbox.set_limittime(limittime)
+            self.passwordinputbox.show(default)
+
+    def close_inputbox(self, symbol):
+        if not self.user_interaction:
+            return
+        self.inputbox.close(symbol)
+        self.passwordinputbox.close(symbol)
+
 
 class BalloonWindow(object):
 
@@ -364,9 +385,14 @@ class BalloonWindow(object):
         self.height = 0
         self.__font_name = ''
         self.text_count = 0
-        self.balloon_pixbuf = None
-        # create drawing area
-        self.darea = gtk.DrawingArea()
+        self.balloon_surface = None
+        self.autoscroll = True
+        self.dragged = False
+        self.x_root = None
+        self.y_root = None
+        self.x_fractions = 0
+        self.y_fractions = 0
+        self.darea = self.window.get_child()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
                               gtk.gdk.BUTTON_PRESS_MASK|
                               gtk.gdk.POINTER_MOTION_MASK|
@@ -374,9 +400,9 @@ class BalloonWindow(object):
                               gtk.gdk.SCROLL_MASK)
         self.darea.connect('expose_event', self.redraw)
         self.darea.connect('button_press_event', self.button_press)
+        self.darea.connect('button_release_event', self.button_release)
         self.darea.connect('motion_notify_event', self.motion_notify)
         self.darea.connect('scroll_event', self.scroll)
-        self.window.add(self.darea)
         self.window.realize()
         self.window.window.set_back_pixmap(None, False)
         self.layout = pango.Layout(self.darea.get_pango_context())
@@ -394,8 +420,12 @@ class BalloonWindow(object):
             text_g = 255 - text_g
             text_b = 255 - text_b
         self.text_active_color = (text_r / 255., text_g / 255., text_b / 255.)
+        sstp_r = desc.get_with_type('sstpmessage.font.color.r', int, text_r)
+        sstp_g = desc.get_with_type('sstpmessage.font.color.g', int, text_g)
+        sstp_b = desc.get_with_type('sstpmessage.font.color.b', int, text_b)
+        self.sstp_message_color = (sstp_r / 255., sstp_g / 255., sstp_b / 255.)
         # initialize
-        self.direction = min(side, 1) ## kluge: multi character
+        self.__direction = min(side, 1) ## kluge: multi character
         self.position = (0, 0)
         self.reset_fonts()
         self.clear_text()
@@ -405,31 +435,36 @@ class BalloonWindow(object):
 
     @property
     def scale(self):
-        scalling = self.request_parent('GET', 'get_preference', 'balloon_scalling')
+        scaling = self.request_parent('GET', 'get_preference', 'balloon_scaling')
         scale = self.request_parent('GET', 'get_preference', 'surface_scale')
-        return scale if scalling else 100 # [%]
+        return scale if scaling else 100 # [%]
+
+    @property
+    def direction(self):
+        return self.__direction
+
+    @direction.setter
+    def direction(self, direction):
+        if self.__direction != direction:
+            self.__direction = direction # 0: left, 1: right
+            self.reset_balloon()
 
     def get_balloon_windowposition(self):
-        winpos_x = self.__get_with_scaling('windowposition.x', int, 0)
-        winpos_y = self.__get_with_scaling('windowposition.y', int, 0)
-        return winpos_x, winpos_y
+        x = self.__get_with_scaling('windowposition.x', int, 0)
+        y = self.__get_with_scaling('windowposition.y', int, 0)
+        return x, y
 
-    def get_pixbuf(self, balloon_id):
-        #assert balloon_id in self.balloon
+    def get_image_surface(self, balloon_id):
+        if balloon_id not in self.balloon:
+            return None
         try:
             path, config = self.balloon[balloon_id]
             use_pna = self.request_parent('GET', 'get_preference', 'use_pna')
-            pixbuf = ninix.pix.create_pixbuf_from_file(
+            surface = ninix.pix.create_surface_from_file(
                 path, use_pna=use_pna)
         except:
             return None
-        scale = self.scale
-        w = pixbuf.get_width()
-        h = pixbuf.get_height()
-        w = max(8, int(w * scale / 100))
-        h = max(8, int(h * scale / 100))
-        pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
-        return pixbuf, (w, h)
+        return surface
 
     def reset_fonts(self):
         font_name = self.request_parent('GET', 'get_preference', 'balloon_fonts')
@@ -443,8 +478,7 @@ class BalloonWindow(object):
                 ['font.height', 'font.size'], int, default_size)
             pango_size = size * 3 / 4 # convert from Windows to GTK+
             pango_size *= pango.SCALE
-        scale = self.scale
-        self.font_desc.set_size(pango_size * scale / 100)
+        self.font_desc.set_size(pango_size)
         self.__font_name = font_name
         self.layout.set_font_description(self.font_desc)
         self.layout.set_wrap(pango.WRAP_CHAR) # XXX
@@ -458,7 +492,7 @@ class BalloonWindow(object):
                     'sstpmessage.font.height', int, default_size)
                 pango_size = size * 3 / 4 # convert from Windows to GTK+
                 pango_size *= pango.SCALE
-            self.sstp_font_desc.set_size(pango_size * scale / 100)
+            self.sstp_font_desc.set_size(pango_size)
             self.sstp_layout.set_font_description(self.sstp_font_desc)
             self.sstp_layout.set_wrap(pango.WRAP_CHAR)
         if self.balloon_id is not None:
@@ -466,17 +500,11 @@ class BalloonWindow(object):
             if self.__shown:
                 self.darea.queue_draw()
 
-    @property
-    def alpha_channel(self):
-        alpha = self.request_parent('GET', 'get_preference', 'balloon_alpha')
-        if alpha is None or not 0.1 <= alpha <= 1.0:
-            alpha = 1.0
-        return alpha
-
     def reset_sstp_marker(self):
         if self.side == 0:
-            assert self.balloon_pixbuf is not None
-            w, h = self.balloon_pixbuf[1]
+            assert self.balloon_surface is not None
+            w = self.balloon_surface.get_width()
+            h = self.balloon_surface.get_height()
             # sstp marker position
             self.sstp = []
             x = self.config_adjust('sstpmarker.x', w,  30)
@@ -485,23 +513,24 @@ class BalloonWindow(object):
             x = self.config_adjust('sstpmessage.x', w,  50)
             y = self.config_adjust('sstpmessage.y', h, -20)
             self.sstp.append((x, y)) # sstp message
-        # sstp marker pixbuf (not only for self.side == 0)
-        self.sstp_pixbuf = self.get_pixbuf('sstp')
+        # sstp marker surface (not only for self.side == 0)
+        self.sstp_surface = self.get_image_surface('sstp')
 
     def reset_arrow(self):
         # arrow positions
         self.arrow = []
-        assert self.balloon_pixbuf is not None
-        w, h = self.balloon_pixbuf[1]
+        assert self.balloon_surface is not None
+        w = self.balloon_surface.get_width()
+        h = self.balloon_surface.get_height()
         x = self.config_adjust('arrow0.x', w, -10)
         y = self.config_adjust('arrow0.y', h,  10)
         self.arrow.append((x, y))
         x = self.config_adjust('arrow1.x', w, -10)
         y = self.config_adjust('arrow1.y', h, -20)
         self.arrow.append((x, y))
-        # arrow pixbufs and sizes
-        self.arrow0_pixbuf = self.get_pixbuf('arrow0')
-        self.arrow1_pixbuf = self.get_pixbuf('arrow1')
+        # arrow surfaces
+        self.arrow0_surface = self.get_image_surface('arrow0')
+        self.arrow1_surface = self.get_image_surface('arrow1')
 
     def reset_message_regions(self):
         w, h = self.layout.get_pixel_size()
@@ -509,26 +538,26 @@ class BalloonWindow(object):
         self.line_space = 1
         self.layout.set_spacing(self.line_space)
         # font metrics
-        origin_x = self.__get_with_scaling(
+        origin_x = self.__get_with_type(
             'origin.x', int,
-            self.__get_with_scaling(
+            self.__get_with_type(
                 'zeropoint.x', int,
-                self.__get_with_scaling('validrect.left', int, 14)))
-        origin_y = self.__get_with_scaling(
+                self.__get_with_type('validrect.left', int, 14)))
+        origin_y = self.__get_with_type(
             'origin.y', int,
-            self.__get_with_scaling(
+            self.__get_with_type(
                 'zeropoint.y', int,
-                self.__get_with_scaling('validrect.top', int, 14)))
-        wpx = self.__get_with_scaling(
-            'wordwrappoint.x', int, 
-            self.__get_with_scaling('validrect.right', int, -14))
+                self.__get_with_type('validrect.top', int, 14)))
+        wpx = self.__get_with_type(
+            'wordwrappoint.x', int,
+            self.__get_with_type('validrect.right', int, -14))
         if wpx > 0:
             line_width = wpx - origin_x
         elif wpx < 0:
             line_width = self.width - origin_x + wpx
         else:
             line_width = self.width - origin_x * 2
-        wpy = self.__get_with_scaling('validrect.bottom', int, -14)
+        wpy = self.__get_with_type('validrect.bottom', int, -14)
         if wpy > 0:
             text_height = min(wpy, self.height) - origin_y
         elif wpy < 0:
@@ -536,7 +565,7 @@ class BalloonWindow(object):
         else:
             text_height = self.height - origin_y * 2
         line_height = self.font_height + self.line_space
-        self.lines = text_height / line_height
+        self.lines = text_height // line_height
         self.line_regions = []
         y = origin_y
         for _ in range(self.lines + 1):
@@ -551,12 +580,12 @@ class BalloonWindow(object):
             self.sstp_region = (x, y, w, h)
 
     def update_line_regions(self, offset, new_y):
-        origin_y = self.__get_with_scaling(
-            'origin.y', int, 
-            self.__get_with_scaling(
+        origin_y = self.__get_with_type(
+            'origin.y', int,
+            self.__get_with_type(
                 'zeropoint.y', int,
-                self.__get_with_scaling('validrect.top', int, 14)))
-        wpy = self.__get_with_scaling('validrect.bottom', int, -14)
+                self.__get_with_type('validrect.top', int, 14)))
+        wpy = self.__get_with_type('validrect.bottom', int, -14)
         if wpy > 0:
             text_height = min(wpy, self.height) - origin_y
         elif wpy < 0:
@@ -565,7 +594,7 @@ class BalloonWindow(object):
             text_height = self.height - origin_y * 2
         line_height = self.font_height + self.line_space
         origin_x, y, line_width, line_height = self.line_regions[offset]
-        self.lines = offset + (text_height - new_y) / line_height
+        self.lines = offset + (text_height - new_y) // line_height
         y = new_y
         for i in range(offset, len(self.line_regions)):
             self.line_regions[i] = (origin_x, y, line_width, line_height)
@@ -574,40 +603,44 @@ class BalloonWindow(object):
             self.line_regions.append((origin_x, y, line_width, line_height))
             y += line_height
 
-    def get_balloon_size(self):
-        return (self.width, self.height)
+    def get_balloon_size(self, scaling=True):
+        w = self.width
+        h = self.height
+        if scaling:
+            scale = self.scale
+            w = int(w * scale / 100.0)
+            h = int(h * scale / 100.0)
+        return w, h
 
     def reset_balloon(self):
         self.set_balloon(self.num)
 
     def set_balloon(self, num):
         self.num = num
-        balloon_id = self.id_format.format(num * 2 + self.direction)
-        self.balloon_pixbuf = self.get_pixbuf(balloon_id)
-        if self.balloon_pixbuf is None:
-            balloon_id = self.id_format.format(0 + self.direction)
-            self.balloon_pixbuf = self.get_pixbuf(balloon_id)
-        assert self.balloon_pixbuf is not None
+        balloon_id = self.id_format.format(int(num * 2 + self.direction))
+        self.balloon_surface = self.get_image_surface(balloon_id)
+        if self.balloon_surface is None:
+            balloon_id = self.id_format.format(int(0 + self.direction))
+            self.balloon_surface = self.get_image_surface(balloon_id)
+        assert self.balloon_surface is not None
         self.balloon_id = balloon_id
-        # change pixbuf and window position
+        # change surface and window position
         x, y = self.position
-        self.width, self.height = self.balloon_pixbuf[1]
-        self.darea.set_size_request(self.width, self.height)
-        mask = gtk.gdk.Pixmap(None, self.width, self.height, 1)
-        self.balloon_pixbuf[0].render_threshold_alpha(
-            mask, 0, 0, 0, 0, self.width, self.height, 1)
-        self.window.mask = mask
+        self.width = self.balloon_surface.get_width()
+        self.height = self.balloon_surface.get_height()
+        scale = self.scale
+        w = int(self.width * scale / 100.0)
+        h = int(self.height * scale / 100.0)
+        self.window.update_size(w, h)
         self.reset_arrow()
         self.reset_sstp_marker()
         self.reset_message_regions()
+        self.request_parent('NOTIFY', 'position_balloons')
         if self.__shown:
-            self.request_parent('NOTIFY', 'position_balloons')
             self.darea.queue_draw()
 
-    def set_direction(self, dir):
-        if self.direction != dir:
-            self.direction = dir # 0: left, 1: right
-            self.reset_balloon()
+    def set_autoscroll(self, flag):
+        self.autoscroll = bool(flag)
 
     def config_adjust(self, name, base, default_value):
         path, config = self.balloon[self.balloon_id]
@@ -618,7 +651,16 @@ class BalloonWindow(object):
             value = default_value
         if value < 0:
             value = base + value
-        return int(value * self.scale / 100)
+        return int(value)
+
+    def __get_with_type(self, name, conv, default_value):
+        path, config = self.balloon[self.balloon_id]
+        value = config.get_with_type(name, conv)
+        if value is None:
+            value = self.desc.get_with_type(name, conv)
+            if value is None:
+                value = default_value
+        return conv(value)
 
     def __get_with_scaling(self, name, conv, default_value):
         path, config = self.balloon[self.balloon_id]
@@ -631,28 +673,30 @@ class BalloonWindow(object):
 
     def __move(self):
         x, y = self.get_position()
-        self.window.resize_move(x, y)
+        self.window.move(x, y)
 
     def set_position(self, base_x, base_y):
         if self.balloon_id is None: ## FIXME
             return
         px, py = self.get_balloon_windowposition()
+        w, h = self.get_balloon_size()
         if self.direction == 0:
-            w, h = self.get_balloon_size()
             x = base_x + px - w
         else:
             x = base_x + px
         y = base_y + py
+        left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
+        if y + h > scrn_h: # XXX
+            y = scrn_h - h
+        if y < top: # XXX
+            y = top
         self.position = (x, y)
-        if self.__shown:
-            self.__move()
+        self.__move()
 
     def get_position(self):
         return self.position
 
     def destroy(self, finalize=0):
-        self.window.remove(self.darea)
-        self.darea.destroy()
         self.window.destroy()
 
     def is_shown(self):
@@ -662,9 +706,10 @@ class BalloonWindow(object):
         if self.__shown:
             return
         self.__shown = 1
-        self.darea.show()
+        # make sure window is in its position (call before showing the window)
+        self.__move()
         self.window.show()
-        # make sure window is in its position
+        # make sure window is in its position (call after showing the window)
         self.__move()
         self.raise_()
 
@@ -708,40 +753,40 @@ class BalloonWindow(object):
         self.sstp_message = None
         self.darea.queue_draw()
 
-    def redraw_sstp_message(self):
+    def redraw_sstp_message(self, widget, cr):
         if self.sstp_message is None:
             return
-        cr = self.darea.window.cairo_create()
+        cr.save()
         # draw sstp marker
-        if self.sstp_pixbuf is not None:
+        if self.sstp_surface is not None:
             x, y = self.sstp[0]
-            cr.set_source_pixbuf(self.sstp_pixbuf[0], x, y)
+            cr.set_source_surface(self.sstp_surface, x, y)
             cr.paint()
         # draw sstp message
         x, y, w, h = self.sstp_region
         self.sstp_layout.set_text(self.sstp_message)
-        cr.set_source_rgba(*self.text_normal_color)
+        cr.set_source_rgba(*self.sstp_message_color)
         cr.move_to(x, y)
         cr.show_layout(self.sstp_layout)
-        del cr
+        cr.restore()
 
-    def redraw_arrow0(self):
+    def redraw_arrow0(self, widget, cr):
         if self.lineno <= 0:
             return
-        cr = self.darea.window.cairo_create()
+        cr.save()
         x, y = self.arrow[0]
-        cr.set_source_pixbuf(self.arrow0_pixbuf[0], x, y)
+        cr.set_source_surface(self.arrow0_surface, x, y)
         cr.paint()
-        del cr
+        cr.restore()
 
-    def redraw_arrow1(self):
+    def redraw_arrow1(self, widget, cr):
         if self.lineno + self.lines >= len(self.text_buffer):
             return
-        cr = self.darea.window.cairo_create()
+        cr.save()
         x, y = self.arrow[1]
-        cr.set_source_pixbuf(self.arrow1_pixbuf[0], x, y)
+        cr.set_source_surface(self.arrow1_surface, x, y)
         cr.paint()
-        del cr
+        cr.restore()
 
     def set_markup(self, index, text):
         tags_ = ('sup', 'sub', 's', 'u')
@@ -777,42 +822,39 @@ class BalloonWindow(object):
                     count_[tag_] -= 1
         return text
 
-    def redraw(self, darea, event):
+    def redraw(self, widget, cr):
         if not self.__shown:
             return True
-        assert self.balloon_pixbuf is not None
-        cr = darea.window.cairo_create()
-        cr.save()
-        cr.set_operator(cairo.OPERATOR_CLEAR)
+        assert self.balloon_surface is not None
+        scale = self.scale
+        cr.scale(scale / 100.0, scale / 100.0)
+        cr.set_source_surface(self.balloon_surface, 0, 0)
+        cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.paint()
-        cr.restore()
-        cr.set_source_pixbuf(self.balloon_pixbuf[0], 0, 0)
-        cr.paint_with_alpha(self.alpha_channel)
+        cr.set_operator(cairo.OPERATOR_OVER) # restore default
         # draw images
         for i in range(len(self.images)):
-            pixbuf, (w, h), (x, y) = self.images[i]
-            scale = self.scale
-            w = max(8, int(w * scale / 100))
-            h = max(8, int(h * scale / 100))
+            image_surface, (x, y) = self.images[i]
+            w = image_surface.get_width()
+            h = image_surface.get_height()
             if x == 'centerx':
-                bw, bh = self.get_balloon_size()
-                x = (bw - w) / 2
+                bw, bh = self.get_balloon_size(scaling=False)
+                x = (bw - w) // 2
             else:
                 try:
                     x = int(x)
                 except:
                     continue
             if y == 'centery':
-                bw, bh = self.get_balloon_size()
-                y = (bh - h) / 2
+                bw, bh = self.get_balloon_size(scaling=False)
+                y = (bh - h) // 2
             else:
                 try:
                     y = int(y)
                 except:
                     continue
-            pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
-            cr.set_source_pixbuf(pixbuf, x, y)
-            cr.paint_with_alpha(self.alpha_channel)
+            cr.set_source_surface(image_surface, x, y)
+            cr.paint()
         # draw text
         i = self.lineno
         j = len(self.text_buffer)
@@ -822,7 +864,7 @@ class BalloonWindow(object):
                 break
             x, y, w, h = self.line_regions[line]
             if self.text_buffer[i].endswith('\n[half]'):
-                new_y = int(y + (self.font_height + self.line_space) / 2)
+                new_y = int(y + (self.font_height + self.line_space) // 2)
                 markup = self.set_markup(i, self.text_buffer[i][:-7])
             else:
                 new_y = int(y + self.font_height + self.line_space)
@@ -832,30 +874,31 @@ class BalloonWindow(object):
             cr.set_source_rgba(*self.text_normal_color)
             cr.move_to(x, y)
             cr.show_layout(self.layout)
-            if self.sstp_pixbuf is not None:
+            if self.sstp_surface is not None:
                 for l, c in self.sstp_marker:
                     if l == i:
-                        mw, mh = self.sstp_pixbuf[1]
+                        mw = self.sstp_surface.get_width()
+                        mh = self.sstp_surface.get_height()
                         self.layout.set_text(self.text_buffer[i][:c])
                         text_w, text_h = self.layout.get_pixel_size()
                         mx = x + text_w
-                        my = y + (self.font_height + self.line_space) / 2
-                        my = my - mh / 2
-                        cr.set_source_pixbuf(self.sstp_pixbuf[0], mx, my)
-                        cr.paint_with_alpha(self.alpha_channel)
+                        my = y + (self.font_height + self.line_space) // 2
+                        my = my - mh // 2
+                        cr.set_source_surface(self.sstp_surface, mx, my)
+                        cr.paint()
             i += 1
             line += 1
-        del cr
         if self.side == 0 and self.sstp_message:
-            self.redraw_sstp_message()
+            self.redraw_sstp_message(widget, cr)
         if self.selection is not None:
-            self.update_link_region(darea, self.selection)
-        self.redraw_arrow0()
-        self.redraw_arrow1()
+            self.update_link_region(widget, cr, self.selection)
+        self.redraw_arrow0(widget, cr)
+        self.redraw_arrow1(widget, cr)
+        self.window.set_shape_surface(self.balloon_surface, 0, 0)
         return False
 
-    def update_link_region(self, darea, index):
-        cr = darea.window.cairo_create()
+    def update_link_region(self, widget, cr, index):
+        cr.save()
         sl = self.link_buffer[index][0]
         el = self.link_buffer[index][2]
         if self.lineno <= sl <= self.lineno + self.lines:
@@ -909,7 +952,7 @@ class BalloonWindow(object):
                 cr.move_to(x, y)
                 cr.set_source_rgba(*self.text_active_color)
                 cr.show_layout(self.layout)
-        del cr
+        cr.restore()
 
     def check_link_region(self, px, py):
         new_selection = None
@@ -956,22 +999,35 @@ class BalloonWindow(object):
             self.selection = new_selection
             return 1 # dirty flag
 
-    def motion_notify(self, darea, event):
+    def motion_notify(self, widget, event):
         if event.is_hint:
             x, y, state = self.darea.window.get_pointer()
         else:
             x, y, state = event.x, event.y, event.state
-        if not self.link_buffer:
-            return True
-        px = int(x)
-        py = int(y)
-        if self.check_link_region(px, py):
-            self.darea.queue_draw()
+        scale = self.scale
+        px, py = self.window.winpos_to_surfacepos(x, y, scale)
+        if self.link_buffer:
+            if self.check_link_region(px, py):
+                widget.queue_draw()
+        if not self.request_parent('GET', 'busy'):
+            if state & gtk.gdk.BUTTON1_MASK:
+                if self.x_root is not None and \
+                        self.y_root is not None:
+                    self.dragged = True
+                    x_delta = (event.x_root - self.x_root) * 100 / scale + self.x_fractions
+                    y_delta = (event.y_root - self.y_root) * 100 / scale + self.y_fractions
+                    self.x_fractions = x_delta - int(x_delta)
+                    self.y_fractions = y_delta - int(y_delta)
+                    self.request_parent(
+                        'NOTIFY', 'update_balloon_offset',
+                        self.side, int(x_delta), int(y_delta))
+                    self.x_root = event.x_root
+                    self.y_root = event.y_root
         return True
 
     def scroll(self, darea, event):
-        px = int(event.x)
-        py = int(event.y)
+        px, py = self.window.winpos_to_surfacepos(
+            int(event.x), int(event.y), self.scale)
         if event.direction == gtk.gdk.SCROLL_UP:
             if self.lineno > 0:
                 self.lineno = max(self.lineno - 2, 0)
@@ -993,10 +1049,11 @@ class BalloonWindow(object):
                                 event.button, click, self.side)
             return True
         # arrows
-        px = int(event.x)
-        py = int(event.y)
+        px, py = self.window.winpos_to_surfacepos(
+            int(event.x), int(event.y), self.scale)
         # up arrow
-        w, h = self.arrow0_pixbuf[1]
+        w = self.arrow0_surface.get_width()
+        h = self.arrow0_surface.get_height()
         x, y = self.arrow[0]
         if x <= px <= x + w and y <= py <= y + h:
             if self.lineno > 0:
@@ -1004,7 +1061,8 @@ class BalloonWindow(object):
                 self.darea.queue_draw()
             return True
         # down arrow
-        w, h = self.arrow1_pixbuf[1]
+        w = self.arrow1_surface.get_width()
+        h = self.arrow1_surface.get_height()
         x, y = self.arrow[1]
         if x <= px <= x + w and y <= py <= y + h:
             if self.lineno + self.lines < len(self.text_buffer):
@@ -1022,6 +1080,19 @@ class BalloonWindow(object):
         # balloon's background
         self.request_parent('NOTIFY', 'notify_balloon_click',
                             event.button, click, self.side)
+        self.x_root = event.x_root
+        self.y_root = event.y_root
+        return True
+
+    def button_release(self, window, event):
+        x, y = self.window.winpos_to_surfacepos(
+            int(event.x), int(event.y), self.scale)
+        if self.dragged:
+            self.dragged = False
+        self.x_root = None
+        self.y_root = None
+        self.y_fractions = 0
+        self.y_fractions = 0
         return True
 
     def clear_text(self):
@@ -1092,7 +1163,7 @@ class BalloonWindow(object):
             i = n
 
     def append_sstp_marker(self):
-        if self.sstp_pixbuf is None:
+        if self.sstp_surface is None:
             return
         if not self.text_buffer:
             line = 0
@@ -1104,7 +1175,8 @@ class BalloonWindow(object):
             line = line + 1
             offset = 0
         self.sstp_marker.append((line, offset))
-        w, h = self.sstp_pixbuf[1]
+        w = self.sstp_surface.get_width()
+        h = self.sstp_surface.get_height()
         i = 1
         while 1:
             space = u'\u3000' * i ## FIXME
@@ -1158,13 +1230,11 @@ class BalloonWindow(object):
 
     def append_image(self, path, x, y):
         try:
-            pixbuf = ninix.pix.create_pixbuf_from_file(path)
+            image_surface = ninix.pix.create_surface_from_file(path)
         except:
             return
         self.show()
-        w = pixbuf.get_width()
-        h = pixbuf.get_height()
-        self.images.append((pixbuf, (w, h), (x, y)))
+        self.images.append((image_surface, (x, y)))
         self.darea.queue_draw()
 
     def draw_last_line(self, column=0):
@@ -1175,28 +1245,30 @@ class BalloonWindow(object):
             x, y, w, h = self.line_regions[line - self.lineno]
             if self.text_buffer[line].endswith('\n[half]'):
                 offset = line - self.lineno + 1
-                new_y = int(y + (self.font_height + self.line_space) / 2)
+                new_y = int(y + (self.font_height + self.line_space) // 2)
                 self.update_line_regions(offset, new_y)
             else:
                 self.darea.queue_draw()
-            if self.sstp_pixbuf is not None:
+            if self.sstp_surface is not None:
                 for l, c in self.sstp_marker:
                     if l == line:
-                        mw, mh = self.sstp_pixbuf[1]
+                        mw = self.sstp_surface.get_width()
+                        mh = self.sstp_surface.get_height()
                         self.layout.set_text(self.text_buffer[l][:c])
                         text_w, text_h = self.layout.get_pixel_size()
                         mx = x + text_w
-                        my = y + (self.font_height + self.line_space) / 2
-                        my = my - mh / 2
+                        my = y + (self.font_height + self.line_space) // 2
+                        my = my - mh // 2
                         cr = self.darea.window.cairo_create()
-                        cr.set_source_pixbuf(self.sstp_pixbuf[0], mx, my)
+                        cr.set_source_surface(self.sstp_surface, mx, my)
                         cr.paint()
                         del cr
         else:
             self.darea.queue_draw()
-            while line >= self.lineno + self.lines:
-                self.lineno += 1
-                self.darea.queue_draw()
+            if self.autoscroll:
+                while line >= self.lineno + self.lines:
+                    self.lineno += 1
+                    self.darea.queue_draw()
 
 
 class CommunicateWindow(object):
@@ -1221,6 +1293,12 @@ class CommunicateWindow(object):
         self.window.connect('delete_event', self.delete)
         self.window.connect('key_press_event', self.key_press)
         self.window.connect('button_press_event', self.button_press)
+        self.window.connect('drag_data_received', self.drag_data_received)
+        # DnD data types
+        dnd_targets = [('text/plain', 0, 0)]
+        self.window.drag_dest_set(gtk.DEST_DEFAULT_ALL, dnd_targets,
+                                  gtk.gdk.ACTION_COPY)
+        self.window.drag_dest_add_text_targets()
         self.window.set_events(gtk.gdk.BUTTON_PRESS_MASK)
         self.window.set_modal(True)
         self.window.set_position(gtk.WIN_POS_CENTER)
@@ -1268,6 +1346,9 @@ class CommunicateWindow(object):
             self.window.add(box)
             box.show()
 
+    def drag_data_received(self, widget, context, x, y, data, info, time):
+        self.entry.set_text(data.get_text())
+
     def redraw(self, darea, event, pixbuf):
         cr = darea.window.cairo_create()
         cr.save()
@@ -1350,7 +1431,8 @@ class CommunicateBox(CommunicateWindow):
         self.send(self.entry.get_text())
 
     def cancel(self):
-        self.send(None)
+        self.request_parent('NOTIFY', 'notify_event',
+                            'OnCommunicateInputCancel', '', 'cancel')
 
     def send(self, data):
         if data:
@@ -1369,7 +1451,9 @@ class TeachBox(CommunicateWindow):
         self.send(self.entry.get_text())
 
     def cancel(self):
-        self.send(None)
+        self.request_parent('NOTIFY', 'notify_event',
+                            'OnTeachInputCancel', '', 'cancel')
+        self.request_parent('NOTIFY', 'reset_user_interaction')
 
     def send(self, data):
         if data:
@@ -1414,15 +1498,23 @@ class InputBox(CommunicateWindow):
 
     def timeout(self):
         self.window.hide()
-        self.send('timeout')
+        self.send('timeout', timeout=True)
 
     def enter(self):
         self.send(self.entry.get_text())
 
     def cancel(self):
-        self.send(None)
+        self.send(None, cancel=True)
 
-    def send(self, data):
+    def close(self, symbol):
+        if self.symbol is None:
+            return
+        if symbol != '__SYSTEM_ALL_INPUT__' and self.symbol != symbol:
+            return
+        self.window.hide()
+        self.cancel()
+
+    def send(self, data, cancel=False, timeout=False):
         if self.timeout_id is not None:
             glib.source_remove(self.timeout_id)            
         if data:
@@ -1430,16 +1522,35 @@ class InputBox(CommunicateWindow):
         if data is None:
             data = ''
         ## CHECK: symbol
-        if self.symbol == 'OnUserInput' and \
+        if cancel:
+            self.request_parent('NOTIFY', 'notify_event',
+                                'OnUserInputCancel', '', 'cancel')
+        elif timeout and \
+                self.request_parent('GET', 'notify_event',
+                                    'OnUserInputCancel', '', 'timeout'):
+            pass
+        elif self.symbol == 'OnUserInput' and \
            self.request_parent('GET', 'notify_event', 'OnUserInput', data):
             pass
-        elif self.request_parent('GET', 'notify_event', 'OnUserInput', self.symbol, data):
+        elif self.request_parent('GET', 'notify_event',
+                                 'OnUserInput', self.symbol, data):
             pass
         elif self.request_parent('GET', 'notify_event', self.symbol, data):
             pass
+        self.symbol = None
         self.request_parent('NOTIFY', 'reset_user_interaction')
 
 
+class PasswordInputBox(InputBox):
+
+    NAME = 'passwordinputbox'
+    ENTRY = 'PasswordInput'
+
+    def new(self, desc, balloon):
+        InputBox.new(self, desc, balloon)
+        self.entry.set_visibility(False)
+
+
 def test():
     pass
 
