@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  hanayu.py - a "花柚" compatible Saori module for ninix
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -445,7 +445,7 @@ class Line(Graph):
                 h = pixbuf.get_height()
                 x = 60 + index * step + step // 2 - w // 2
                 y = 220 - int(
-                    168 * self.args[index * 2 + 1] / (limit_max - limit_min)) - h / 2
+                    168 * self.args[index * 2 + 1] / (limit_max - limit_min)) - h // 2
                 cr.set_source_pixbuf(pixbuf, x, y)
                 cr.paint()
         del cr
