@@ -3,7 +3,7 @@
 #  kawari.py - a "華和梨" compatible Shiori module for ninix
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -1196,7 +1196,7 @@ class Kawari(object):
                     value *= int(tree[i + 1])
                 elif tree[i] == '/':
                     try:
-                        value /= int(tree[i + 1])
+                        value //= int(tree[i + 1])
                     except ZeroDivisionError:
                         raise ExprError
                 elif tree[i] == '%':
@@ -1701,14 +1701,14 @@ class Shiori(Kawari):
             'Shiori: KAWARI compatible module for ninix\n'
             '        Copyright (C) 2001, 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
-            '        Copyright (C) 2002-2012 by Shyouzou Sugitani\n'
+            '        Copyright (C) 2002-2013 by Shyouzou Sugitani\n'
             '        Copyright (C) 2003 by Shun-ichi TAHARA')
 
     def request(self, req_string):
         header = req_string.splitlines()
         req_header = {}
-        line = header.pop(0)
-        if line:
+        if header:
+            line = header.pop(0)
             line = line.strip()
             req_list = line.split()
             if len(req_list) >= 2:
@@ -1822,8 +1822,8 @@ class Shiori(Kawari):
         response = self.saori_request(self.saori_ini[alias][0],
                                       req.encode(charset, 'ignore'))
         header = response.splitlines()
-        line = header.pop(0)
-        if line:
+        if header:
+            line = header.pop(0)
             line = line.strip()
             if ' ' in line:
                 saori_protocol, saori_statuscode = [x.strip() for x in line.split(' ', 1)]
@@ -1872,8 +1872,8 @@ class Shiori(Kawari):
         response = self.saori_request(self.saori_ini[alias][0],
                                       req.encode(charset, 'ignore'))
         header = response.splitlines()
-        line = header.pop(0)
-        if line:
+        if header:
+            line = header.pop(0)
             line = line.strip()
             if ' ' in line:
                 saori_protocol, saori_statuscode = [x.strip() for x in line.split(' ', 1)]
