@@ -3,7 +3,7 @@
 #  niseshiori.py - a "偽栞" compatible Shiori module for ninix
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -305,9 +305,9 @@ class NiseShiori(object):
                                    'kero.recommendsites',
                                    'sakura.portalsites']:
                         id_list = self.resources.get(argv[0], '')
-	                if id_list:
+                        if id_list:
                             id_list = ''.join((id_list, '\2'))
-	                id_list = ''.join((id_list, argv[1].replace(' ', '\1')))
+                        id_list = ''.join((id_list, argv[1].replace(' ', '\1')))
                         self.resources[argv[0]] = id_list
                     else:
                         self.resources[argv[0]] = argv[1]
@@ -830,7 +830,7 @@ class NiseShiori(object):
                 if tree[i] == '*':
                     value = int(value) * int(operand)
                 elif tree[i] == '/':
-                    value = int(value) / int(operand)
+                    value = int(value) // int(operand)
                 elif tree[i] == '\\':
                     value = int(value) % int(operand)
             except (ValueError, ZeroDivisionError):
@@ -1030,7 +1030,7 @@ class Shiori(NiseShiori):
             'Shiori: NiseShiori compatible module for ninix\n'
             '        Copyright (C) 2001, 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
-            '        Copyright (C) 2002-2012 by Shyouzou Sugitani\n'
+            '        Copyright (C) 2002-2013 by Shyouzou Sugitani\n'
             '        Copyright (C) 2003 by Shun-ichi TAHARA')
 
     def request(self, req_string):
