@@ -3,7 +3,7 @@
 #  satori.py - a "里々" compatible Shiori module for ninix
 #  Copyright (C) 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003, 2004 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -54,7 +54,7 @@ NODE_UNARY_EXPR = 26
 def encrypt(s):
     buf = []
     t = len(s)
-    p = (t + 1) / 2
+    p = (t + 1) // 2
     for n in range(p):
         buf.append(s[n])
         if len(s[p:]) > n:
@@ -1562,7 +1562,7 @@ class Satori(object):
 
     def make_wait(self, ms):
         buf = []
-        n = (ms + 25) * self.wait_percent / 100 / 50
+        n = int((ms + 25) * self.wait_percent / 100 / 50)
         while n > 0:
             buf.append(r'\w{0:d}'.format(min(n, 9)))
             n -= 9
@@ -1765,7 +1765,7 @@ class Satori(object):
                     if operand is None:
                         operand = 0
                     if node[i] == '/':
-                        value /= operand
+                        value //= operand
                     elif node[i] == '%':
                         value %= operand
                     else:
@@ -2133,24 +2133,24 @@ class Satori(object):
             return self.DAYOFWEEK[now[6]]
         runtime = self.get_runtime()
         if s == u'起動時':
-            return self.to_zenkaku(runtime / 3600)
+            return self.to_zenkaku(runtime // 3600)
         elif s == u'起動分':
-            return self.to_zenkaku(runtime / 60 % 60)
+            return self.to_zenkaku(runtime // 60 % 60)
         elif s == u'起動秒':
             return self.to_zenkaku(runtime % 60)
         elif s == u'単純起動分':
-            return self.to_zenkaku(runtime / 60)
+            return self.to_zenkaku(runtime // 60)
         elif s == u'単純起動秒':
             return self.to_zenkaku(runtime)
         accumulative_runtime = self.runtime + self.get_runtime()
         if s == u'累計時':
-            return self.to_zenkaku(accumulative_runtime / 3600)
+            return self.to_zenkaku(accumulative_runtime // 3600)
         elif s == u'累計分':
-            return self.to_zenkaku(accumulative_runtime / 60 % 60)
+            return self.to_zenkaku(accumulative_runtime // 60 % 60)
         elif s == u'累計秒':
             return self.to_zenkaku(accumulative_runtime % 60)
         elif s == u'単純累計分':
-            return self.to_zenkaku(accumulative_runtime / 60)
+            return self.to_zenkaku(accumulative_runtime // 60)
         elif s == u'単純累計秒':
             return self.to_zenkaku(accumulative_runtime)
         elif s == u'起動回数':
@@ -2183,7 +2183,7 @@ class Satori(object):
         elif s == 'Event':
             return self.event
         elif s == 'Charset':
-            return 'EUC-JP'
+            return 'UTF-8'
         elif s.startswith('Reference'):
             n = int(s[9:])
             if self.reference[n] is None:
@@ -2251,17 +2251,17 @@ class Shiori(Satori):
             'Shiori: SATORI compatible module for ninix\n'
             '        Copyright (C) 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
-            '        Copyright (C) 2002-2012 by Shyouzou Sugitani\n'
+            '        Copyright (C) 2002-2013 by Shyouzou Sugitani\n'
             '        Copyright (C) 2003, 2004 by Shun-ichi TAHARA')
 
     def request(self, req_string):
         if self.folder_change:
             self.change_folder()
             self.folder_change = 0
-        header = unicode(req_string, 'EUC-JP').splitlines()
+        header = unicode(req_string, 'UTF-8').splitlines()
         req_header = {}
-        line = header.pop(0)
-        if line:
+        if header:
+            line = header.pop(0)
             line = line.strip()
             req_list = line.split()
             if len(req_list) >= 2:
@@ -2321,12 +2321,12 @@ class Shiori(Satori):
             self.silent_time = 0
         result = 'SHIORI/3.0 200 OK\r\n' \
                  'Sender: Satori\r\n' \
-                 'Charset: EUC-JP\r\n' \
+                 'Charset: UTF-8\r\n' \
                  u'Value: {0}\r\n'.format(result)
         if to is not None:
             result = ''.join((result, u'Reference0: {0}\r\n'.format(to)))
         result = ''.join((result, '\r\n'))
-        return result.encode('EUC-JP') ## FIXME
+        return result.encode('UTF-8') ## FIXME
 
     def call_saori(self, name, args, history, side):
         if name not in self.saori_function or \
@@ -2354,8 +2354,8 @@ class Shiori(Satori):
             self.saori_function[name][0],
             req.encode('Shift_JIS', 'ignore'))
         header = response.splitlines()
-        line = header.pop(0)
-        if line:
+        if header:
+            line = header.pop(0)
             line = line.strip()
             if ' ' in line:
                 saori_protocol, saori_statuscode = [x.strip() for x in line.split(' ', 1)]
