@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  ssu.py - a ssu compatible Saori module for ninix
-#  Copyright (C) 2003-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -24,7 +24,7 @@ from ninix.dll import SAORI
 class Saori(SAORI):
 
     def __init__(self):
-	SAORI.__init__(self)
+        SAORI.__init__(self)
         self.function = {'is_empty':      [self.ssu_is_empty,      [0, 1]],
                          'is_digit':      [self.ssu_is_digit,      [1]],
                          'is_alpha':      [self.ssu_is_alpha,      [1]],
@@ -55,32 +55,36 @@ class Saori(SAORI):
     def request(self, req):
         req_type, argument, charset = self.evaluate_request(req)
         if not req_type:
-            return self.RESPONSE[400]
+            result = self.RESPONSE[400]
         elif req_type == 'GET Version':
-            return self.RESPONSE[204]
+            result = self.RESPONSE[204]
         elif req_type == 'EXECUTE':
-            if argument[0] not in self.function:
-                return self.RESPONSE[400]
-            name = argument[0]
-            argument = argument[1:]
-            if self.function[name][1] == [None]:
-                pass
-            elif len(argument) not in self.function[name][1]:
-                return self.RESPONSE[400]
-            self.value = []
-            result = self.function[name][0](argument, charset)
-            if result is not None and result != '':
-                s = 'SAORI/1.0 200 OK\r\nResult: {0}\r\n'.format(result)
-                if self.value:
-                    for i in range(len(self.value)):
-                        s = ''.join((s, 'Value{0:d}: {1}\r\n'.format(i, self.value[i])))
-                s = ''.join((s, 'Charset: {0}\r\n'.format(charset)))
-                s = ''.join((s, '\r\n'))
-                return s
-            else:
-                return self.RESPONSE[204]
+            result = self.execute(argument, charset)
         else:
+            result = self.RESPONSE[400]
+        return result
+
+    def execute(self, args, charset):
+        if args[0] not in self.function:
             return self.RESPONSE[400]
+        name = args[0]
+        args = args[1:]
+        if self.function[name][1] == [None]:
+            pass
+        elif len(args) not in self.function[name][1]:
+            return self.RESPONSE[400]
+        self.value = []
+        result = self.function[name][0](args, charset)
+        if result is not None and result != '':
+            s = 'SAORI/1.0 200 OK\r\nResult: {0}\r\n'.format(result)
+            if self.value:
+                for i in range(len(self.value)):
+                    s = ''.join((s, 'Value{0:d}: {1}\r\n'.format(i, self.value[i])))
+            s = ''.join((s, 'Charset: {0}\r\n'.format(charset)))
+            s = ''.join((s, '\r\n'))
+            return s
+        else:
+            return self.RESPONSE[204]
 
     def ssu_is_empty(self, args, charset):
         return 1 if not args else 0
