@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  wmove.py - a wmove.dll compatible Saori module for ninix
-#  Copyright (C) 2003-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -40,13 +40,13 @@ class Saori(SAORI):
         if not self.__sakura:
             pass
         elif self.loaded:
-	    result = 2
+            result = 2
         else:
             self.sakura_name = self.__sakura.get_selfname()
             self.kero_name = self.__sakura.get_keroname()
             self.loaded = 1
             result = 1
-	return result
+        return result
 
     def finalize(self):
         if self.timeout_id:
@@ -90,47 +90,51 @@ class Saori(SAORI):
         elif req_type == 'GET Version':
             result = self.RESPONSE[204]
         elif req_type == 'EXECUTE':
-            if not argument:
-                result = self.RESPONSE[400]
-            elif not self.__check_argument(argument):
-                result = self.RESPONSE[400]
-            else:
-                name = argument[0]
-                if name == 'GET_POSITION':
-                    if argument[1] == self.sakura_name: ## FIXME: HWND
-                        side = 0
-                    elif argument[1] == self.kero_name: ## FIXME: HWND
-                        side = 1
-                    else:
-                        return self.RESPONSE[400]
-                    try:
-                        x, y = self.__sakura.get_surface_position(side)
-                        w, h = self.__sakura.get_surface_size(side)
-                        result = 'SAORI/1.0 200 OK\r\n' \
-                            'Result: {0:d}\r\n' \
-                            'Value0: {1:d}\r\n' \
-                            'Value1: {2:d}\r\n' \
-                            'Value2: {3:d}\r\n\r\n'.format(x, x, x + w / 2, x + w)
-                    except:
-                        result = self.RESPONSE[500]
-                elif name == 'GET_DESKTOP_SIZE':
-	            try:
-                        left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-                        result = 'SAORI/1.0 200 OK\r\n' \
-                            'Result: {0:d}\r\n' \
-                            'Value0: {1:d}\r\n' \
-                            'Value1: {2:d}\r\n\r\n'.format(scrn_w, scrn_w, scrn_h)
-	            except:
-                        result = self.RESPONSE[500]
-                else:
-                    self.enqueue_commands(name, argument[1:])
-                    if self.timeout_id is None:
-                        self.do_idle_tasks()
-                    result = self.RESPONSE[204]
+            result = self.execute(argument)
         else:
             result = self.RESPONSE[400]
         return result
 
+    def execute(self, args):
+        if not args:
+            result = self.RESPONSE[400]
+        elif not self.__check_argument(args):
+            result = self.RESPONSE[400]
+        else:
+            name = args[0]
+            if name == 'GET_POSITION':
+                if args[1] == self.sakura_name: ## FIXME: HWND
+                    side = 0
+                elif args[1] == self.kero_name: ## FIXME: HWND
+                    side = 1
+                else:
+                    return self.RESPONSE[400]
+                try:
+                    x, y = self.__sakura.get_surface_position(side)
+                    w, h = self.__sakura.get_surface_size(side)
+                    result = 'SAORI/1.0 200 OK\r\n' \
+                        'Result: {0:d}\r\n' \
+                        'Value0: {1:d}\r\n' \
+                        'Value1: {2:d}\r\n' \
+                        'Value2: {3:d}\r\n\r\n'.format(x, x, x + w // 2, x + w)
+                except:
+                    result = self.RESPONSE[500]
+            elif name == 'GET_DESKTOP_SIZE':
+                try:
+                    left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
+                    result = 'SAORI/1.0 200 OK\r\n' \
+                        'Result: {0:d}\r\n' \
+                        'Value0: {1:d}\r\n' \
+                        'Value1: {2:d}\r\n\r\n'.format(scrn_w, scrn_w, scrn_h)
+                except:
+                    result = self.RESPONSE[500]
+            else:
+                self.enqueue_commands(name, args[1:])
+                if self.timeout_id is None:
+                    self.do_idle_tasks()
+                result = self.RESPONSE[204]
+        return result
+
     def enqueue_commands(self, command, args):
         #assert command in ['MOVE', 'MOVE_INSIDE', 'MOVETO', 'MOVETO_INSIDE',
         #                   'ZMOVE', 'WAIT', 'NOTIFY',
@@ -142,7 +146,7 @@ class Saori(SAORI):
             side = 1
         else:
             return # XXX
-	if command == 'CLEAR':
+        if command == 'CLEAR':
             self.commands[side] = []
         else:
             if command in ['STANDBY', 'STANDBY_INSIDE']:
@@ -151,79 +155,79 @@ class Saori(SAORI):
             self.commands[side].append([command, args[1:]])
 
     def do_idle_tasks(self):
-	for side in [0, 1]:
+        for side in [0, 1]:
             if self.commands[side]:
-	        command, args = self.commands[side].pop(0)
-	        if command in ['MOVE', 'MOVE_INSIDE']:
+                command, args = self.commands[side].pop(0)
+                if command in ['MOVE', 'MOVE_INSIDE']:
                     x, y = self.__sakura.get_surface_position(side)
-	            vx = int(args[0])
-	            speed = int(args[1])
-	            if command == 'MOVE_INSIDE':
+                    vx = int(args[0])
+                    speed = int(args[1])
+                    if command == 'MOVE_INSIDE':
                         w, h = self.__sakura.get_surface_size(side)
                         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-	                if vx < 0 and x + vx <0:
-	                    vx = min(-x, 0)
-	                elif vx > 0 and x + vx + w > left + scrn_w:
-	                    vx = max(left + scrn_w - w - x, 0)
-	            if abs(vx) > speed:
-	                if vx > 0:
-	                    self.__sakura.set_surface_position(
+                        if vx < 0 and x + vx <0:
+                            vx = min(-x, 0)
+                        elif vx > 0 and x + vx + w > left + scrn_w:
+                            vx = max(left + scrn_w - w - x, 0)
+                    if abs(vx) > speed:
+                        if vx > 0:
+                            self.__sakura.set_surface_position(
                                 side, x + speed, y)
-	                    self.commands[side].insert(
+                            self.commands[side].insert(
                                 0, [command, [str(vx - speed), args[1]]])
-	                elif vx < 0:
-	                    self.__sakura.set_surface_position(
+                        elif vx < 0:
+                            self.__sakura.set_surface_position(
                                 side, x - speed, y)
-	                    self.commands[side].insert(
+                            self.commands[side].insert(
                                 0, [command, [str(vx + speed), args[1]]])
-	            else:
-	                self.__sakura.set_surface_position(side, x + vx, y)
-	        elif command in ['MOVETO', 'MOVETO_INSIDE']:
+                    else:
+                        self.__sakura.set_surface_position(side, x + vx, y)
+                elif command in ['MOVETO', 'MOVETO_INSIDE']:
                     x, y = self.__sakura.get_surface_position(side)
-	            to = int(args[0])
-	            speed = int(args[1])
-	            if command == 'MOVETO_INSIDE':
+                    to = int(args[0])
+                    speed = int(args[1])
+                    if command == 'MOVETO_INSIDE':
                         w, h = self.__sakura.get_surface_size(side)
                         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-	                if to < 0:
-	                    to = 0
-	                elif to > left + scrn_w - w:
-	                    to = left + scrn_w - w
-	            if abs(to - x) > speed:
-	                if to - x > 0:
-	                    self.__sakura.set_surface_position(
+                        if to < 0:
+                            to = 0
+                        elif to > left + scrn_w - w:
+                            to = left + scrn_w - w
+                    if abs(to - x) > speed:
+                        if to - x > 0:
+                            self.__sakura.set_surface_position(
                                 side, x + speed, y)
-	                    self.commands[side].insert(0, [command, args])
-	                elif to - x < 0:
-	                    self.__sakura.set_surface_position(
+                            self.commands[side].insert(0, [command, args])
+                        elif to - x < 0:
+                            self.__sakura.set_surface_position(
                                 side, x - speed, y)
-	                    self.commands[side].insert(0, [command, args])
-	            else:
-	                self.__sakura.set_surface_position(side, to, y)
-	        elif command in ['STANDBY', 'STANDBY_INSIDE']:
-	            pass ## FIXME
-	        elif command == 'ZMOVE':
-	            if args[0] == '1':
-	                self.__sakura.raise_surface(side)
-	            elif args[0] == '2':
-	                self.__sakura.lower_surface(side)
-	            else:
-	                pass
-	        elif command == 'WAIT':
-	            try:
-	                wait = int(args[0]) # ms
-	            except:
-	                wait = 0
+                            self.commands[side].insert(0, [command, args])
+                    else:
+                        self.__sakura.set_surface_position(side, to, y)
+                elif command in ['STANDBY', 'STANDBY_INSIDE']:
+                    pass ## FIXME
+                elif command == 'ZMOVE':
+                    if args[0] == '1':
+                        self.__sakura.raise_surface(side)
+                    elif args[0] == '2':
+                        self.__sakura.lower_surface(side)
+                    else:
+                        pass
+                elif command == 'WAIT':
+                    try:
+                        wait = int(args[0]) # ms
+                    except:
+                        wait = 0
                     if wait < 25:
-	                pass
-	            else:
-	                self.commands[side].insert(0, ['WAIT', str(wait - 20)])
-	        elif command == 'NOTIFY':
+                        pass
+                    else:
+                        self.commands[side].insert(0, ['WAIT', str(wait - 20)])
+                elif command == 'NOTIFY':
                     self.__sakura.notify_event(*args)
         if not self.commands[0] and not self.commands[1]:
-	    if self.timeout_id is not None:
+            if self.timeout_id is not None:
                 glib.source_remove(self.timeout_id)
                 self.timeout_id = None
         else:
-	    if self.timeout_id is None:
+            if self.timeout_id is None:
                 self.timeout_id = glib.timeout_add(20, self.do_idle_tasks)
