@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -341,6 +341,8 @@ def find_makoto_dll(top_dir):
 
 def find_surface_set(top_dir):
     desc = read_descript_txt(os.path.join(top_dir, 'ghost', 'master'))
+    default_sakura = desc.get('sakura.seriko.defaultsurface', '0')
+    default_kero = desc.get('kero.seriko.defaultsurface', '10')
     shell_name = desc.get('name') if desc else None
     if not shell_name:
         inst = read_install_txt(top_dir)
@@ -350,13 +352,14 @@ def find_surface_set(top_dir):
     shell_dir = os.path.join(top_dir, 'shell')
     for name, desc, subdir in find_surface_dir(shell_dir):
         surface_dir = os.path.join(shell_dir, subdir)
-        surface_info, alias, tooltips = read_surface_info(surface_dir)
+        surface_info, alias, tooltips, seriko_descript = read_surface_info(surface_dir)
         if surface_info and \
-           'surface0' in surface_info and 'surface10' in surface_info:
+                'surface{}'.format(default_sakura) in surface_info and \
+                'surface{}'.format(default_kero) in surface_info:
             if alias is None:
                 alias = read_alias_txt(surface_dir)
             surface_set[subdir] = (name, surface_dir, desc, alias,
-                                   surface_info, tooltips)
+                                   surface_info, tooltips, seriko_descript)
     return shell_name, surface_set
 
 def find_surface_dir(top_dir):
@@ -424,6 +427,7 @@ def read_surface_info(surface_dir):
     # find surfaces.txt
     alias = None
     tooltips = {}
+    seriko_descript = {}
     for key, config in read_surfaces_txt(surface_dir):
         if key == '__alias__':
             alias = config
@@ -437,6 +441,8 @@ def read_surface_info(surface_dir):
             except KeyError:
                 img = None
             surface[key] = (img, config)
+        elif key == 'descript':
+            seriko_descript = config
     # find surface elements
     for key, (img, config) in surface.items():
         for key, method, filename, x, y in list_surface_elements(config):
@@ -445,7 +451,9 @@ def read_surface_info(surface_dir):
             if basename not in surface:
                 surface[basename] = (os.path.join(surface_dir, filename),
                                      ninix.config.null_config())
-    return surface, alias, tooltips
+    return surface, alias, tooltips, seriko_descript
+
+re_alias = re.compile('^(sakura|kero|char[0-9]+)\.surface\.alias$')
 
 def read_surfaces_txt(surface_dir):
     config_list = []
@@ -492,7 +500,8 @@ def read_surfaces_txt(surface_dir):
                     if not opened:
                         logging.error(
                             'syntax error: unbalnced "}" in surfaces.txt.')
-                    if key in ['sakura.surface.alias', 'kero.surface.alias']:
+                    match = re_alias.match(key)
+                    if match:
                         alias_buffer.append(key)
                         alias_buffer.append('{')
                         alias_buffer.extend(buf)
@@ -524,6 +533,8 @@ def read_surfaces_txt(surface_dir):
                                 except ValueError:
                                     pass
                             config_list.append((key, ninix.config.create_from_buffer(buf)))
+                    elif key == 'descript':
+                        config_list.append((key, ninix.config.create_from_buffer(buf)))
                     buf = []
                     key = None
                     opened = False
@@ -690,7 +701,8 @@ def test():
                 if alias:
                     buf = []
                     for k, v in alias.items():
-                        if k in ['sakura.surface.alias', 'kero.surface.alias']:
+                        match = re_alias.match(k)
+                        if match:
                             print ''.join((k, ':'))
                             for alias_id, alias_list in v.items():
                                 print alias_id, \
