@@ -3,7 +3,7 @@
 #  install.py - an installer module for ninix
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -172,10 +172,10 @@ class Installer(object):
         filetype = self.get_file_type(tmpdir)
         try:
             func = getattr(self, 'install_{0}'.format(filetype))
-            target_dir = func(filename, tmpdir, homedir)
+            target_dirs = func(filename, tmpdir, homedir)
         finally:
             shutil.rmtree(tmpdir)
-        return filetype, target_dir
+        return filetype, target_dirs
 
 
     def download(self, url, basedir):
@@ -237,7 +237,7 @@ class Installer(object):
         path = os.path.abspath(target_dir)
         if not os.path.isdir(path):
             return
-        os.path.walk(path, self.remove_files, mask)
+        os.walk(path, self.remove_files, mask)
         dirlist = self.list_all_directories(path, '')
         dirlist.sort()
         dirlist.reverse()
@@ -303,7 +303,7 @@ class Installer(object):
         if response != gtk.RESPONSE_ACCEPT:
             return None
         model, it = ts.get_selected()
-        return model.get_value(it, 1)
+        return candidates[model.get_value(it, 0)]
 
     def install_ghost(self, archive, tmpdir, homedir):
         # find install.txt
@@ -390,7 +390,9 @@ class Installer(object):
             except IOError as e:
                 code, message = e.args
                 logging.error('cannot write {0}'.format(path))
-        return target_dir
+        if balloon_dir is None: # XXX
+            balloon_dir = ''
+        return (target_dir, balloon_dir)
 
     def install_supplement(self, archive, tmpdir, homedir):
         inst = ninix.home.read_install_txt(tmpdir)
