@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2004-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -242,10 +242,7 @@ class Skin(object):
         self.__shown = False
         self.surface_id = 0 # dummy
         self.window = ninix.pix.TransparentWindow()
-        self.window.set_focus_on_map(False)
         ##self.window.set_title(''.join(('surface.', name)))
-        self.window.set_decorated(False)
-        self.window.set_resizable(False)
         self.window.set_skip_taskbar_hint(True)
         self.window.connect('delete_event', self.delete)
         self.window.realize()
@@ -266,21 +263,15 @@ class Skin(object):
         self.seriko.set_responsible(self.handle_request)
         path = os.path.join(self.data['dir'], self.data['base'])
         try:
-            self.pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            w = max(8, self.pixbuf.get_width() * self.__scale / 100)
-            h = max(8, self.pixbuf.get_height() * self.__scale / 100)
-            surface_pixbuf = self.pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR)
-            mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
-            surface_pixbuf.render_threshold_alpha(
-                mask_pixmap, 0, 0, 0, 0, w, h, 1)
+            self.image_surface = ninix.pix.create_surface_from_file(path)
+            w = max(8, int(self.image_surface.get_width() * self.__scale / 100))
+            h = max(8, int(self.image_surface.get_height() * self.__scale / 100))
         except: ## FIXME
             self.request_parent('NOTIFY', 'close')
             return
+        self.path = path
         self.w, self.h = w, h
-        self.current_surface_pixbuf = surface_pixbuf
-        self.window.mask = mask_pixmap
-        self.darea = gtk.DrawingArea()
+        self.darea = self.window.get_child()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
                                gtk.gdk.BUTTON_PRESS_MASK|
                                gtk.gdk.BUTTON_RELEASE_MASK|
@@ -291,9 +282,7 @@ class Skin(object):
         self.darea.connect('motion_notify_event', self.motion_notify)
         self.darea.connect('leave_notify_event', self.leave_notify)
         self.darea.connect('expose_event', self.redraw)
-        self.darea.set_size_request(self.w, self.h)
-        self.darea.show()
-        self.window.add(self.darea)
+        self.window.update_size(self.w, self.h)
         self.set_position()
         target_window = self.request_parent('GET', 'get_target_window')
         if self.data['ontop']:
@@ -329,11 +318,11 @@ class Skin(object):
             'GET', 'get_kinoko_position', self.data['baseposition'])
         a, b = [(0.5, 1), (0.5, 0), (0, 0.5), (1, 0.5), (0, 1),
                 (1, 1), (0, 0), (1, 0), (0.5, 0.5)][self.data['baseadjust']]
-        offsetx = self.data['offsetx'] * self.__scale / 100
-        offsety = self.data['offsety'] * self.__scale / 100
+        offsetx = int(self.data['offsetx'] * self.__scale / 100)
+        offsety = int(self.data['offsety'] * self.__scale / 100)
         self.x = base_x - int(self.w * a) + offsetx + xoffset
         self.y = base_y - int(self.h * b) + offsety + yoffset
-        self.window.resize_move(self.x, self.y)
+        self.window.move(self.x, self.y)
 
     def set_scale(self, scale):
         self.__scale = scale
@@ -343,101 +332,80 @@ class Skin(object):
     def get_surface(self): ## FIXME
         return None
 
-    def redraw(self, darea, event):
-        cr = darea.window.cairo_create()
-        cr.save()
-        cr.set_operator(cairo.OPERATOR_CLEAR)
+    def redraw(self, widget, cr):
+        scale = self.__scale
+        cr.scale(scale / 100.0, scale / 100.0)
+        cr.set_source_surface(self.image_surface, 0, 0)
+        cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.paint()
-        cr.restore()
-        cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
-        ##cr.paint_with_alpha(self.__alpha_channel)
-        cr.paint()
-        del cr
+        self.window.set_shape_surface(self.image_surface, 0, 0)
 
-    def get_pixbuf(self, surface_id):
-        path = os.path.join(self.data['dir'], ''.join(('surface', str(surface_id), '.png')))
+    def get_image_surface(self, surface_id):
+        path = os.path.join(self.data['dir'],
+                            ''.join(('surface', str(surface_id), '.png')))
         if os.path.exists(path):
-            pixbuf = ninix.pix.create_pixbuf_from_file(path)
+            surface = ninix.pix.create_surface_from_file(path)
         else:
-            pixbuf = None
-        return pixbuf
+            surface = None
+        return surface
 
-    def create_surface_pixbuf(self, surface_id=None):
+    def create_image_surface(self, surface_id):
         if surface_id is not None and surface_id != '':
-            surface_pixbuf = self.get_pixbuf(surface_id)
+            surface = self.get_image_surface(surface_id)
         else:
-            surface_pixbuf = self.pixbuf.copy()
-        return surface_pixbuf
+            surface = ninix.pix.create_surface_from_file(self.path)
+        return surface
 
     def update_frame_buffer(self):
-        surface_pixbuf = self.create_surface_pixbuf(self.seriko.base_id)
-        assert surface_pixbuf is not None
+        new_surface = self.create_image_surface(self.seriko.base_id)
+        assert new_surface is not None
         # draw overlays
-        for pixbuf_id, x, y in self.seriko.iter_overlays():
+        for surface_id, x, y, method in self.seriko.iter_overlays():
             try:
-                pixbuf = self.get_pixbuf(pixbuf_id)
-                w = pixbuf.get_width()
-                h = pixbuf.get_height()
+                overlay_surface = self.get_image_surface(surface_id)
             except:
                 continue
-            # overlay surface pixbuf
-            sw = surface_pixbuf.get_width()
-            sh = surface_pixbuf.get_height()
-            if x + w > sw:
-                w = sw - x
-            if y + h > sh:
-                h = sh - y
-            if x < 0:
-                dest_x = 0
-                w += x
-            else:
-                dest_x = x
-            if y < 0:
-                dest_y = 0
-                h += y
-            else:
-                dest_y = y
-            pixbuf.composite(surface_pixbuf, dest_x, dest_y,
-                             w, h, x, y, 1.0, 1.0,
-                             gtk.gdk.INTERP_BILINEAR, 255)
-        w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
-        h = max(8, surface_pixbuf.get_height() * self.__scale / 100)
-        surface_pixbuf = surface_pixbuf.scale_simple(
-            w, h, gtk.gdk.INTERP_BILINEAR)
-        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
-        surface_pixbuf.render_threshold_alpha(
-            mask_pixmap, 0, 0, 0, 0, w, h, 1)
+            # overlay surface
+            cr = cairo.Context(new_surface)
+            cr.set_source_surface(overlay_surface, x, y)
+            cr.mask_surface(overlay_surface, x, y)
+            del cr
         #self.darea.queue_draw_area(0, 0, w, h)
-        self.window.mask = mask_pixmap
-        self.current_surface_pixbuf = surface_pixbuf
+        self.image_surface = new_surface
         self.darea.queue_draw()
 
     def terminate(self):
         self.seriko.terminate(self)
 
-    def add_overlay(self, actor, surface_id, x, y):
-        self.seriko.add_overlay(self, actor, surface_id, x, y)
+    def add_overlay(self, actor, surface_id, x, y, method):
+        self.seriko.add_overlay(self, actor, surface_id, x, y, method)
 
     def remove_overlay(self, actor):
         self.seriko.remove_overlay(actor)
 
     def move_surface(self, xoffset, yoffset):
-        self.window.resize_move(self.x + xoffset, self.y + yoffset)
+        self.window.move(self.x + xoffset, self.y + yoffset)
 
     def reset_surface(self): ## FIXME
         self.seriko.reset(self, '') # XXX
         path = os.path.join(self.data['dir'], self.data['base'])
         w, h = ninix.pix.get_png_size(path)
-        w = max(8, w * self.__scale / 100)
-        h = max(8, h * self.__scale / 100)
-        self.w, self.h = w, h # XXX            
-        self.darea.set_size_request(w, h)
+        w = max(8, int(w * self.__scale / 100))
+        h = max(8, int(h * self.__scale / 100))
+        self.w, self.h = w, h # XXX
+        self.window.update_size(w, h)
         self.window.queue_resize()
         self.seriko.start(self)
         self.seriko.invoke_kinoko(self)
 
     def set_surface(self, surface_id, restart=1): ## FIXME
-        self.pixbuf = self.get_pixbuf(surface_id)
+        path = os.path.join(self.data['dir'],
+                            ''.join(('surface', str(surface_id), '.png')))
+        if os.path.exists(path):
+            self.path = path
+        else:
+            #self.path = None
+            self.path = os.path.join(self.data['dir'], self.data['base'])
 
     def invoke(self, actor_id, update=0):
         self.seriko.invoke(self, actor_id, update)
