@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2003-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -273,9 +273,22 @@ class Menu(object):
 
     def create_mayuna_menu(self, mayuna_menu):
         self.__mayuna_menu = []
-        for side, index in [('sakura', 0), ('kero', 1)]:
+        for side in mayuna_menu:
+            if side == 'sakura':
+                index = 0
+            elif side == 'kero':
+                index = 1
+            elif side.startswith('char'):
+                try:
+                    index = int(side[4:])
+                except:
+                    continue
+            else:
+                continue
+            for _ in range(len(self.__mayuna_menu), index + 1):
+                self.__mayuna_menu.append(None)
             if mayuna_menu[side]:
-                self.__mayuna_menu.append(gtk.Menu())
+                self.__mayuna_menu[index] = gtk.Menu()
                 item = gtk.TearoffMenuItem()
                 item.show()
                 self.__mayuna_menu[index].append(item)
@@ -294,8 +307,6 @@ class Menu(object):
                         item = gtk.SeparatorMenuItem()
                     item.show()
                     self.__mayuna_menu[index].append(item)
-            else:
-                self.__mayuna_menu.append(None)
 
     __re_shortcut = re.compile(r'&(?=[\x21-\x7e])')
 
@@ -490,20 +501,43 @@ class Menu(object):
                     if title == '-':
                         item = gtk.SeparatorMenuItem()
                     else:
-                        if len(entry) < 2:
-                            continue
-                        url = entry[1]
-                        ## FIXME
-                        #if len(entry) > 2:
-                        #    bannar = entry[2]
-                        #else:
-                        #    bannar = None
                         item = gtk.MenuItem(title)
-                        item.connect(
-                            'activate',
-                            lambda a, i: self.request_parent(
-                                'NOTIFY', 'notify_site_selection', i),
-                            (title, url))
+                        if len(entry) < 2:
+                            item.set_sensitive(False)
+                        if len(entry) > 1:
+                            url = entry[1]
+                        if len(entry) > 2:
+                            base_path = self.request_parent(
+                                'GET', 'get_prefix')
+                            filename = entry[2].lower()
+                            head, tail = os.path.splitext(filename)
+                            if not tail:
+                                for ext in ['.png', '.jpg', '.gif']:
+                                    filename = ''.join((filename, ext))
+                                    banner = os.path.join(
+                                        base_path, 'ghost/master/banner',
+                                        filename)
+                                    if not os.path.exists(banner):
+                                        banner = None
+                                    else:
+                                        break
+                            else:
+                                banner = os.path.join(
+                                    base_path, 'ghost/master/banner',
+                                    filename)
+                                if not os.path.exists(banner):
+                                    banner = None
+                        else:
+                            banner = None
+                        if len(entry) > 1:
+                            item.connect(
+                                'activate',
+                                lambda a, i: self.request_parent(
+                                    'NOTIFY', 'notify_site_selection', i),
+                                (title, url))
+                            item.set_has_tooltip(True)
+                            item.connect('query-tooltip',
+                                         self.on_tooltip, banner)
                     menu.add(item)
                     item.show()
                 menuitem = self.ui_manager.get_widget(
@@ -526,20 +560,41 @@ class Menu(object):
                 if title == '-':
                     item = gtk.SeparatorMenuItem()
                 else:
-                    if len(entry) < 2:
-                        continue
-                    url = entry[1]
-                    ## FIXME
-                    #if len(entry) > 2:
-                    #    bannar = entry[2]
-                    #else:
-                    #    bannar = None
                     item = gtk.MenuItem(title)
-                    item.connect(
-                        'activate',
-                        lambda a, i: self.request_parent(
-                            'NOTIFY', 'notify_site_selection', i),
-                        (title, url))
+                    if len(entry) < 2:
+                        item.set_sensitive(False)
+                    if len(entry) > 1:
+                        url = entry[1]
+                    if len(entry) > 2:
+                        base_path = self.request_parent('GET', 'get_prefix')
+                        filename = entry[2].lower()
+                        head, tail = os.path.splitext(filename)
+                        if not tail:
+                            for ext in ['.png', '.jpg', '.gif']:
+                                filename = ''.join((filename, ext))
+                                banner = os.path.join(
+                                    base_path, 'ghost/master/banner',
+                                    filename)
+                                if not os.path.exists(banner):
+                                    banner = None
+                                else:
+                                    break
+                        else:
+                            banner = os.path.join(
+                                base_path, 'ghost/master/banner',
+                                filename)
+                            if not os.path.exists(banner):
+                                banner = None
+                    else:
+                        banner = None
+                    if len(entry) > 1:
+                        item.connect(
+                            'activate',
+                            lambda a, i: self.request_parent(
+                                'NOTIFY', 'notify_site_selection', i),
+                            (title, url))
+                        item.set_has_tooltip(True)
+                        item.connect('query-tooltip', self.on_tooltip, banner)
                 menu.add(item)
                 item.show()
             menuitem =  self.ui_manager.get_widget(
@@ -550,7 +605,7 @@ class Menu(object):
         else:
             self.__set_visible('Recommend', 0)
 
-    def create_ghost_menuitem(self, name, icon, key, handler):
+    def create_ghost_menuitem(self, name, icon, key, handler, thumbnail):
         if icon is not None:
             pixbuf = ninix.pix.create_icon_pixbuf(icon)
             if pixbuf is None:
@@ -567,8 +622,17 @@ class Menu(object):
         item.set_name('popup menu item')
         item.show()
         item.connect('activate', lambda a, v: handler(v), (key))
+        item.set_has_tooltip(True)
+        item.connect('query-tooltip', self.on_tooltip, thumbnail)
         return item
 
+    def on_tooltip(self, widget, x, y, keyboard_mode, tooltip, thumbnail):
+        if thumbnail is None:
+            return False
+        pixbuf = ninix.pix.create_pixbuf_from_file(thumbnail)
+        tooltip.set_icon(pixbuf)
+        return True
+
     def __set_ghost_menu(self):
         for path in ['Summon', 'Change']:
             ghost_menu = gtk.Menu()
@@ -604,11 +668,13 @@ class Menu(object):
                 menu.append(item)
         return menu
 
-    def create_meme_menuitem(self, name, value, handler):
+    def create_meme_menuitem(self, name, value, handler, thumbnail):
         item = gtk.MenuItem(name)
         item.set_name('popup menu item')
         item.show()
         item.connect('activate', lambda a, v: handler(v), (value))
+        item.set_has_tooltip(True)
+        item.connect('query-tooltip', self.on_tooltip, thumbnail)
         return item
 
     def __set_plugin_menu(self):
