@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2004-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -260,15 +260,12 @@ class Skin(object):
         self.window = ninix.pix.TransparentWindow()
         name, top_dir = ninix.home.read_profile_txt(dir) # XXX
         self.window.set_title(name)
-        self.window.set_decorated(False)
-        self.window.set_resizable(False)
-        self.window.set_focus_on_map(False)
         self.window.connect('delete_event', self.delete)
         self.window.connect('key_press_event', self.key_press)
         self.window.realize()
         self.window.window.set_back_pixmap(None, False)
         self.window.add_accel_group(self.accelgroup)
-        self.darea = gtk.DrawingArea()
+        self.darea = self.window.get_child()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
                               gtk.gdk.BUTTON_PRESS_MASK|
                               gtk.gdk.BUTTON_RELEASE_MASK|
@@ -281,9 +278,6 @@ class Skin(object):
         self.darea.connect('motion_notify_event', self.motion_notify)
         self.darea.connect('leave_notify_event', self.leave_notify)
         self.id = [0, None]
-        self.darea.show() ## FIXME
-        self.window.add(self.darea)
-        self.darea.realize()
         self.set_surface()
         self.set_position(reset=1)
         self.window.show()
@@ -309,16 +303,13 @@ class Skin(object):
         self.set_surface()
         self.set_position()
 
-    def redraw(self, darea, event):
-        cr = darea.window.cairo_create()
-        cr.save()
-        cr.set_operator(cairo.OPERATOR_CLEAR)
+    def redraw(self, widget, cr):
+        scale = self.__scale
+        cr.scale(scale / 100.0, scale / 100.0)
+        cr.set_source_surface(self.image_surface, 0, 0)
+        cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.paint()
-        cr.restore()
-        cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
-        ##cr.paint_with_alpha(self.__alpha_channel)
-        cr.paint()
-        del cr
+        self.window.set_shape_surface(self.image_surface, 0, 0)
 
     def delete(self, widget, event):
         self.request_parent('NOTIFY', 'finalize')
@@ -361,21 +352,15 @@ class Skin(object):
             path = os.path.join(self.dir,
                                 'surface{0:d}.png'.format(self.id[0]))
         try:
-            surface_pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
-            h = max(8, surface_pixbuf.get_height() * self.__scale / 100)
-            surface_pixbuf = surface_pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR)
+            new_surface = ninix.pix.create_surface_from_file(path)
+            w = max(8, int(new_surface.get_width() * self.__scale / 100))
+            h = max(8, int(new_surface.get_height() * self.__scale / 100))
         except:
             self.request_parent('NOTIFY', 'finalize')
             return
         self.w, self.h = w, h
-        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
-        surface_pixbuf.render_threshold_alpha(
-            mask_pixmap, 0, 0, 0, 0, w, h, 1)
-        self.darea.set_size_request(self.w, self.h)
-        self.window.mask = mask_pixmap
-        self.current_surface_pixbuf = surface_pixbuf
+        self.window.update_size(self.w, self.h)
+        self.image_surface = new_surface
         self.darea.queue_draw()
 
     def set_position(self, reset=0):
@@ -386,7 +371,7 @@ class Skin(object):
         else:
             if not self.omni:
                 self.y = top + scrn_h - self.h
-        self.window.resize_move(self.x, self.y)
+        self.window.move(self.x, self.y)
 
     def move(self, x_delta, y_delta):
         self.x = self.x + x_delta
@@ -419,11 +404,12 @@ class Skin(object):
         self.y_root = None
         return True
 
-    def motion_notify(self,  widget, event):
+    def motion_notify(self, widget, event):
         if event.is_hint:
             x, y, state = self.darea.window.get_pointer()
         else:
             x, y, state = event.x, event.y, event.state
+        #x, y = self.window.winpos_to_surfacepos(x, y, self.__scale)
         if state & gtk.gdk.BUTTON1_MASK:
             if self.x_root is not None and \
                self.y_root is not None:
@@ -509,16 +495,13 @@ class Katochan(object):
     def delete(self, widget, event):
         self.destroy()
 
-    def redraw(self, darea, event):
-        cr = darea.window.cairo_create()
-        cr.save()
-        cr.set_operator(cairo.OPERATOR_CLEAR)
+    def redraw(self, widget, cr):
+        scale = self.__scale
+        cr.scale(scale / 100.0, scale / 100.0)
+        cr.set_source_surface(self.image_surface, 0, 0)
+        cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.paint()
-        cr.restore()
-        cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
-        ##cr.paint_with_alpha(self.__alpha_channel)
-        cr.paint()
-        del cr
+        self.window.set_shape_surface(self.image_surface, 0, 0)
 
     def set_movement(self, timing):
         key = ''.join((timing, 'fall.type'))
@@ -558,30 +541,24 @@ class Katochan(object):
         target_x, target_y = self.target.get_surface_position(self.side)
         target_w, target_h = self.target.get_surface_size(self.side)
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-        self.x = target_x + target_w / 2 - self.w / 2 + \
-                 self.offset_x * self.__scale / 100
-        self.y = top + self.offset_y * self.__scale / 100
-        self.window.resize_move(self.x, self.y)
+        self.x = target_x + target_w // 2 - self.w // 2 + \
+                 int(self.offset_x * self.__scale / 100)
+        self.y = top + int(self.offset_y * self.__scale / 100)
+        self.window.move(self.x, self.y)
 
     def set_surface(self):
         path = os.path.join(self.data['dir'], 'surface{0:d}.png'.format(self.id))
         try:
-            surface_pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
-            h = max(8, surface_pixbuf.get_height() * self.__scale / 100)
-            surface_pixbuf = surface_pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR)
+            new_surface = ninix.pix.create_surface_from_file(path)
+            w = max(8, int(new_surface.get_width() * self.__scale / 100))
+            h = max(8, int(new_surface.get_height() * self.__scale / 100))
         except:
             self.request_parent('NOTIFY', 'finalize')
             return
         self.w, self.h = w, h
-        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
-        surface_pixbuf.render_threshold_alpha(
-            mask_pixmap, 0, 0, 0, 0, w, h, 1)
-        self.darea.set_size_request(self.w, self.h)
+        self.window.update_size(self.w, self.h)
         #self.darea.queue_draw_area(0, 0, self.w, self.h)
-        self.window.mask = mask_pixmap
-        self.current_surface_pixbuf = surface_pixbuf
+        self.image_surface = new_surface
         self.darea.queue_draw()
 
     def load(self, data):
@@ -633,17 +610,11 @@ class Katochan(object):
         self.offset_y = offset_y
         self.window = ninix.pix.TransparentWindow()
         self.window.set_title(self.data['name'])
-        self.window.set_decorated(False)
-        self.window.set_resizable(False)
         self.window.set_skip_taskbar_hint(True) # XXX
-        self.window.set_focus_on_map(False)
         self.window.connect('delete_event', self.delete)
-        self.darea = gtk.DrawingArea()
+        self.darea = self.window.get_child()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
         self.darea.connect('expose_event', self.redraw)
-        self.darea.show() ## FIXME
-        self.window.add(self.darea)
-        self.darea.realize()
         self.window.realize()
         self.window.window.set_back_pixmap(None, False)
         self.window.show()
@@ -679,14 +650,14 @@ class Katochan(object):
                 pass ## FIXME
             else:
                 pass
-        self.window.resize_move(self.x, self.y)
+        self.window.move(self.x, self.y)
 
     def check_collision(self): ## FIXME: check self position
         for side in [0, 1]:
             target_x, target_y = self.target.get_surface_position(side)
             target_w, target_h = self.target.get_surface_size(side)
-            center_x = self.x + self.w / 2
-            center_y = self.y + self.h / 2
+            center_x = self.x + self.w // 2
+            center_y = self.y + self.h // 2
             if target_x < center_x < target_x + target_w and \
                target_y < center_y < target_y + target_h:
                 self.side = side
@@ -696,10 +667,10 @@ class Katochan(object):
 
     def check_mikire(self):
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-        if self.x + self.w - self.w / 3 > left + scrn_w or \
-                self.x + self.w / 3 < left or \
-                self.y + self.h - self.h / 3 > top + scrn_h or \
-                self.y + self.h / 3 < top:
+        if self.x + self.w - self.w // 3 > left + scrn_w or \
+                self.x + self.w // 3 < left or \
+                self.y + self.h - self.h // 3 > top + scrn_h or \
+                self.y + self.h // 3 < top:
             return 1
         else:
             return 0
