@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001-2004 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2004-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -478,7 +478,7 @@ class UI(object):
                 pass ## FIXME
 
     def show_next(self):
-        self.request_parent('NOTIFY', 'next')
+        self.request_parent('NOTIFY', 'go_next')
         self.update()
 
     def show_previous(self):
@@ -735,14 +735,14 @@ class NGM(object):
     def show_all(self):
         pass
 
-    def next(self):
-        next = self.current + 1
-        if next < ID_LIMIT and next in self.catalog.data:
-            self.current = next
+    def go_next(self):
+        next_index = self.current + 1
+        if next_index < ID_LIMIT and next_index in self.catalog.data:
+            self.current = next_index
 
     def exist_next(self):
-        next = self.current + 1
-        return bool(next < ID_LIMIT and next in self.catalog.data)
+        next_index = self.current + 1
+        return bool(next_index < ID_LIMIT and next_index in self.catalog.data)
 
     def previous(self):
         previous = self.current - 1
