@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2003-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -17,55 +17,17 @@ import logging
 
 import numpy
 import gtk
+import cairo
 
 
-class TransparentWindow(gtk.Window):
+class BaseTransparentWindow(gtk.Window):
     __gsignals__ = {'screen-changed': 'override', }
 
     def __init__(self, type=gtk.WINDOW_TOPLEVEL):
-        gtk.Window.__init__(self, type)
-        self.set_app_paintable(True)
+        gtk.Window.__init__(self, type=type)
+        self.set_decorated(False)
+        self.set_resizable(False)
         self.screen_changed()
-        self.truncated = False
-        self.mask_offset = (0, 0)
-        fixed = gtk.Fixed()
-        fixed.show()
-        gtk.Window.add(self, fixed) # XXX
-        self.__fixed = fixed
-        self.__mask = None
-        self.__child = None
-        self.__position = (0, 0)
-        self.connect_after('size_allocate', self.size_allocate)
-
-    @property
-    def mask(self):
-        return self.__mask
-
-    @mask.setter
-    def mask(self, mask):
-        x, y = self.mask_offset
-        self.__mask = mask
-        if self.is_composited():
-            self.input_shape_combine_mask(mask, x, y)
-        else:
-            self.shape_combine_mask(mask, x, y)
-
-    @property
-    def fixed(self): # read only
-        return self.__fixed
-
-    def add(self, widget):
-        self.fixed.put(widget, 0, 0)
-        self.__child = widget
-
-    def remove(self, widget):
-        self.fixed.remove(widget)
-        self.__child = None
-
-    def destroy(self):
-        gtk.Window.remove(self, self.fixed)
-        self.fixed.destroy()
-        gtk.Window.destroy(self)
 
     def screen_changed(self, old_screen=None):
         screen = self.get_screen()
@@ -78,66 +40,92 @@ class TransparentWindow(gtk.Window):
             self.supports_alpha = False
         self.set_colormap(colormap)
 
+
+class TransparentWindow(BaseTransparentWindow):
+
+    def __init__(self, type=gtk.WINDOW_TOPLEVEL):
+        BaseTransparentWindow.__init__(self, type=type)
+        self.set_app_paintable(True)
+        self.set_focus_on_map(False)
+        self.__position = (0, 0)
+        self.__surface_position = (0, 0)
+        self.__redraw = None
+        self.connect_after('size_allocate', self.size_allocate)
+        # create drawing area
+        self.darea = gtk.DrawingArea()
+        self.darea.show()
+        self.darea.connect = self.wrap_connect
+        self.add(self.darea)
+
+    def wrap_connect(self, signal, user_function, user_data=None):
+        if signal in ['expose_event', 'expose-vent']:
+            self.__redraw = (user_function, user_data)
+            gtk.DrawingArea.connect(self.darea, signal, self.wrap_draw)
+        else:
+            if user_data is not None:
+                gtk.DrawingArea.connect(
+                    self.darea, signal, user_function, user_data)
+            else:
+                gtk.DrawingArea.connect(
+                    self.darea, signal, user_function)
+
+    def wrap_draw(self, darea, event):
+        if self.__redraw is None:
+            return
+        cr = darea.window.cairo_create()
+        cr.translate(*self.get_draw_offset())
+        user_function, user_data = self.__redraw
+        if user_data is not None:
+            user_function(darea, cr, user_data)
+        else:
+            user_function(darea, cr)
+
+    def set_shape_surface(self, surface, x, y):
+        w, h = self.get_child().get_size_request() # XXX
+        bitmap = gtk.gdk.Pixmap(None, w, h, 1)
+        cr = bitmap.cairo_create()
+        cr.set_source_rgb(0.0, 0.0, 0.0)
+        cr.set_operator(cairo.OPERATOR_CLEAR)
+        cr.paint()
+        cr.translate(*self.get_draw_offset())
+        cr.set_operator(cairo.OPERATOR_SOURCE)
+        cr.set_source_surface(surface, x, y)
+        cr.paint()
+        del cr
+        if self.is_composited():
+            self.input_shape_combine_mask(bitmap, 0, 0)
+        else:
+            self.shape_combine_mask(bitmap, 0, 0)
+
+    def update_size(self, w, h):
+        self.get_child().set_size_request(w, h) # XXX
+        self.queue_resize()
+
     def size_allocate(self, widget, event):
         new_x, new_y = self.__position
         gtk.Window.move(self, new_x, new_y)
 
-    def reset_truncate(self):
-        self.fixed.move(self.__child, 0, 0)
-        self.set_size_request(-1, -1)
-        if self.is_composited():
-            self.input_shape_combine_mask(self.mask, 0, 0)
-        else:
-            self.shape_combine_mask(self.mask, 0, 0)
-        self.mask_offset = (0, 0)
-        self.truncated = False
-
-    def resize_move(self, x, y, xoffset=0, yoffset=0):
-        ### XXX: does not work propery for gtk.WINDOW_POPUP
-        ##if self.type == gtk.WINDOW_POPUP:
-        ##    gtk.Window.move(self, x, y)
-        ##    return
-        w, h = self.__child.get_size_request()
+    def move(self, x, y):
         left, top, scrn_w, scrn_h = get_workarea()
-        new_x = min(max(x + xoffset, left - w + 1), left + scrn_w - 1)
-        new_y = min(max(y + yoffset, top - h + 1), top + scrn_h - 1)
-        new_w = w
-        new_h = h
-        offset_x = 0
-        offset_y = 0
-        truncate = False
-        if new_x < left:
-            offset_x = new_x - left
-            new_x = left
-            new_w += offset_x
-            truncate = True
-        if new_x + w > left + scrn_w:
-            new_w -= new_x + w - (left + scrn_w)
-            truncate = True
-        if new_y < top:
-            offset_y = new_y - top
-            new_y = top
-            new_h += offset_y
-            truncate = True
-        if new_y + h > top + scrn_h:
-            new_h -= new_y + h - (top + scrn_h)
-            truncate = True
-        if not truncate:
-            if self.truncated:
-                self.reset_truncate()
-            else:
-                gtk.Window.move(self, new_x, new_y) # XXX
-        else:
-            self.fixed.move(self.__child, offset_x, offset_y)
-            self.set_size_request(new_w, new_h)
-            if self.is_composited():
-                self.input_shape_combine_mask(self.mask, offset_x, offset_y)
-            else:
-                self.shape_combine_mask(self.mask, offset_x, offset_y)
-            self.truncated = True
-            self.mask_offset = (offset_x, offset_y)
+        w, h = self.get_child().get_size_request() # XXX
+        new_x = min(max(left, x), left + scrn_w - w)
+        new_y = min(max(top, y), top + scrn_h - h)
+        gtk.Window.move(self, new_x, new_y)
         self.__position = (new_x, new_y)
-        self.__child.queue_draw()
+        self.__surface_position = (x, y)
+        self.darea.queue_draw()
+
+    def get_draw_offset(self):
+        window_x, window_y = self.__position
+        surface_x, surface_y = self.__surface_position
+        return surface_x - window_x, surface_y - window_y
+
+    def winpos_to_surfacepos(self, x, y, scale):
+        window_x, window_y = self.__position
+        surface_x, surface_y = self.__surface_position
+        new_x = int((x - (surface_x - window_x)) * 100 / scale)
+        new_y = int((y - (surface_y - window_y)) * 100 / scale)
+        return new_x, new_y
 
 
 def get_png_size(path):
@@ -163,7 +151,7 @@ def get_png_size(path):
 def get_DGP_IHDR(path):
     head, tail = os.path.split(path)
     filename = tail
-    m_half = hashlib.md5(filename[:len(filename) / 2]).hexdigest()
+    m_half = hashlib.md5(filename[:len(filename) // 2]).hexdigest()
     m_full = hashlib.md5(filename).hexdigest()
     tmp = ''.join((m_full, filename))
     key = ''
@@ -213,6 +201,11 @@ def __pixbuf_new_from_file(path):
         path = unicode(path, 'mbcs').encode('utf-8')
     return gtk.gdk.pixbuf_new_from_file(path)
 
+def __surface_new_from_file(path):
+    if os.name == 'nt': # XXX
+        path = unicode(path, 'mbcs').encode('utf-8')
+    return cairo.ImageSurface.create_from_png(path)
+
 def create_icon_pixbuf(path):
     try:
         pixbuf = __pixbuf_new_from_file(path)
@@ -227,10 +220,14 @@ def create_blank_pixbuf(width, height):
     pixbuf.fill(0xffffffffL)
     return pixbuf
 
+def create_blank_surface(width, height): ## FIXME
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
+    return surface
+
 def create_pixbuf_from_DGP_file(path):
     head, tail = os.path.split(path)
     filename = tail
-    m_half = hashlib.md5(filename[:len(filename) / 2]).hexdigest()
+    m_half = hashlib.md5(filename[:len(filename) // 2]).hexdigest()
     m_full = hashlib.md5(filename).hexdigest()
     tmp = ''.join((m_full, filename))
     key = ''
@@ -306,6 +303,20 @@ def create_pixbuf_from_file(path, is_pnr=True, use_pna=False):
             pixbuf_array[:,:,3] = pna_array[:,:,0]
     return pixbuf
 
+def create_surface_from_file(path, is_pnr=True, use_pna=False):
+    head, tail = os.path.split(path)
+    basename, ext = os.path.splitext(tail)
+    pixbuf = create_pixbuf_from_file(path, is_pnr, use_pna)
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32,
+                                 pixbuf.get_width(), pixbuf.get_height())
+    cr = cairo.Context(surface)
+    cr = gtk.gdk.CairoContext(cr)
+    cr.set_source_pixbuf(pixbuf, 0, 0)
+    cr.set_operator(cairo.OPERATOR_SOURCE)
+    cr.paint()
+    del cr
+    return surface
+
 def create_pixmap_from_file(path):
     pixbuf = create_pixbuf_from_file(path)
     pixmap, mask = pixbuf.render_pixmap_and_mask(1)
