@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -121,13 +121,13 @@ class Controler(object):
             pass
         self.__dirty = 1
 
-    def add_overlay(self, window, actor, pixbuf_id, x, y):
-        if pixbuf_id == '-2':
+    def add_overlay(self, window, actor, surface_id, x, y, method):
+        if surface_id == '-2':
             self.terminate(window)
-        if pixbuf_id in ['-1', '-2']:
+        if surface_id in ['-1', '-2']:
             self.remove_overlay(actor)
             return
-        self.overlays[actor] = (pixbuf_id, x, y)
+        self.overlays[actor] = (surface_id, x, y, method)
         self.__dirty = 1
 
     def invoke_actor(self, window, actor):
@@ -185,7 +185,8 @@ class Controler(object):
         for actor in self.seriko[self.base_id]:
             interval = actor.get_interval()
             if interval in ['always', 'sometimes', 'rarely'] or \
-               interval.startswith('random'):
+               interval.startswith('random') or \
+               interval.startswith('periodic'):
                 self.invoke_actor(window, actor)
 
     def invoke_restart(self, window):
@@ -232,6 +233,13 @@ class Controler(object):
         self.__move = None
         self.__dirty = 1
 
+    def stop_actor(self, actor_id):
+        if self.base_id not in self.seriko:
+            return
+        for actor in self.seriko[self.base_id]:
+            if actor.get_id() == actor_id:
+                actor.terminate()
+
     def destroy(self):
         if self.timeout_id is not None:
             glib.source_remove(self.timeout_id)
@@ -243,11 +251,11 @@ class Controler(object):
         actors.sort()
         actors[:] = [actor for actor_id, actor in actors]
         for actor in actors:
-            pixbuf_id, x, y = self.overlays[actor]
+            surface_id, x, y, method = self.overlays[actor]
             ##logging.debug(
             ##    'actor={0:d}, id={1}, x={2:d}, y={3:d}'.format(
-            ##        actor.get_id(), pixbuf_id, x, y))
-            yield pixbuf_id, x, y
+            ##        actor.get_id(), surface_id, x, y))
+            yield surface_id, x, y, method
 
 
 class Actor(object):
@@ -302,19 +310,26 @@ class Actor(object):
                 surface_ids.append(surface)
         return surface_ids
 
+    OVERLAY_SET = {'overlay', 'overlayfast',
+                   'interpolate', 'reduce', 'replace', 'asis'}
+
     def show_pattern(self, window, surface, method, args):
-        if self.last_method in ['overlay', 'overlayfast']:
+        if self.last_method in self.OVERLAY_SET:
             window.remove_overlay(self)
         if method == 'move':
             window.seriko.move_surface(args[0], args[1]) ## FIXME
-        elif method in ['overlay', 'overlayfast']:
-            window.add_overlay(self, surface, args[0], args[1])
+        elif method in self.OVERLAY_SET:
+            window.add_overlay(self, surface, args[0], args[1], method)
         elif method == 'base':
             window.seriko.set_base_id(window, surface) ## FIXME
         elif method == 'start':
             window.invoke(args[0], update=1)
         elif method == 'alternativestart':
             window.invoke(random.choice(args), update=1)
+        elif method == 'stop':
+            window.seriko.stop_actor(args[0]) ## FIXME
+        elif method == 'alternativestop':
+            window.seriko.stop_actor(random.choice(args)) ## FIXME
         else:
             raise RuntimeError, 'should not reach here'
         self.last_method = method
@@ -347,7 +362,7 @@ class ActiveActor(Actor): # always
         return False
 
 
-class RandomActor(Actor): # sometimes, rarely, randome
+class RandomActor(Actor): # sometimes, rarely, random, periodic
 
     def __init__(self, actor_id, interval, wait_min, wait_max):
         Actor.__init__(self, actor_id, interval)
@@ -458,11 +473,10 @@ re_seriko_interval_value = re.compile('^(sometimes|rarely|random,[0-9]+|always|r
 re_seriko_pattern = re.compile(r'^([0-9]+|-[12])\s*,\s*([+-]?[0-9]+)\s*,\s*(overlay|overlayfast|base|move|start|alternativestart|)\s*,?\s*([+-]?[0-9]+)?\s*,?\s*([+-]?[0-9]+)?\s*,?\s*(\[[0-9]+(\.[0-9]+)*\])?$')
 
 re_seriko2_interval = re.compile('^animation([0-9]+)\.interval$')
-re_seriko2_interval_value = re.compile('^(sometimes|rarely|random,[0-9]+|always|runonce|yesn-e|talk,[0-9]+|never)$')
-re_seriko2_pattern = re.compile(r'^(overlay|overlayfast|base|move|start|alternativestart|)\s*,\s*([0-9]+|-[12])\s*,\s*([+-]?[0-9]+)\s*,?\s*([+-]?[0-9]+)?\s*,?\s*([+-]?[0-9]+)?\s*,?\s*(\([0-9]+(\.[0-9]+)*\))?$')
+re_seriko2_interval_value = re.compile('^(sometimes|rarely|random,[0-9]+|periodic,[0-9]+|always|runonce|yesn-e|talk,[0-9]+|never)$')
+re_seriko2_pattern = re.compile(r'^(overlay|overlayfast|interpolate|reduce|replace|asis|base|move|start|alternativestart|stop|alternativestop)\s*,\s*([0-9]+|-[12])?\s*,?\s*([+-]?[0-9]+)?\s*,?\s*([+-]?[0-9]+)?\s*,?\s*([+-]?[0-9]+)?\s*,?\s*(\([0-9]+([\.\,][0-9]+)*\))?$')
 
-def get_actors(config):
-    version = None
+def get_actors(config, version=1):
     buf = []
     for key, value in config.items():
         if version == 1:
@@ -470,16 +484,7 @@ def get_actors(config):
         elif version == 2:
             match = re_seriko2_interval.match(key)
         else:
-            match1 = re_seriko_interval.match(key)
-            match2 = re_seriko2_interval.match(key)
-            if match1:
-                version = 1
-                match = match1
-            elif match2:
-                version = 2
-                match = match2
-            else:
-                continue
+            return [] ## should not reach here
         if not match:
             continue
         if version == 1 and not re_seriko_interval_value.match(value):
@@ -496,7 +501,12 @@ def get_actors(config):
         elif interval == 'rarely':
             actor = RandomActor(actor_id, interval, 20000, 60000)
         elif interval.startswith('random'):
-            actor = RandomActor(actor_id, interval, 0, 1000 * int(interval[7]))
+            actor = RandomActor(actor_id, interval,
+                                0, 1000 * int(interval[7:]))
+        elif interval.startswith('periodic'):
+            actor = RandomActor(actor_id, interval,
+                                1000 * int(interval[9:]),
+                                1000 * int(interval[9:]))
         elif interval == 'runonce':
             actor = OneTimeActor(actor_id, interval)
         elif interval == 'yen-e':
@@ -534,20 +544,27 @@ def get_actors(config):
                     method = match.group(3)
                 else:
                     method = match.group(1)
-                    surface = str(int(match.group(2)))
-                    interval = abs(int(match.group(3)))
+                    surface = str(int(match.group(2) or 0))
+                    interval = abs(int(match.group(3) or 0))
                 if method == '':
                     method = 'base'
-                if method == 'start':
-                    group = match.group(4)
+                if method in {'start', 'stop'}:
+                    if version == 2:
+                        group = match.group(2)
+                        surface = -1 # XXX
+                    else:
+                        group = match.group(4)
                     if group is None:
                         raise ValueError, 'syntax error: {0}'.format(pattern)
                     args = [int(group)]
-                elif method == 'alternativestart':
+                elif method in {'alternativestart', 'alternativestop'}:
                     args = match.group(6)
                     if args is None:
                         raise ValueError, 'syntax error: {0}'.format(pattern)
-                    args = [int(s) for s in args[1:-1].split('.')]
+                    t = []
+                    for x in args[1:-1].split('.'):
+                        t.extend(x.split(','))
+                    args = [int(s) for s in t]
                 else:
                     if surface in ['-1', '-2']:
                         x = 0
