@@ -3,7 +3,7 @@
 #  sstplib.py - an SSTP library module in Python
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -123,7 +123,7 @@ class BaseSSTPRequestHandler(SocketServer.StreamRequestHandler):
         t = time.localtime(time.time())
         m = self.month_names[t[1] - 1]
         return '{0:02d}/{1}/{2:d}:{3:02d}:{4:02d}:{5:02d} {6:+05d}'.format(
-            t[2], m, t[0], t[3], t[4], t[5], -time.timezone / 36)
+            t[2], m, t[0], t[3], t[4], t[5], -time.timezone // 36)
 
 
 def test(ServerClass = SSTPServer,
