@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -57,7 +57,8 @@ class NetworkUpdate(object):
     def start(self, homeurl, ghostdir, timeout=60):
         url = urlparse.urlparse(homeurl)
         if not (url[0] == 'http' and url[3] == url[4] == url[5] == ''):
-            self.enqueue_event('OnUpdateFailure', 'bad home URL')
+            self.enqueue_event('OnUpdateFailure', 'bad home URL', '', '',
+                               'ghost') # XXX
             self.state = None
             return
         try:
@@ -74,7 +75,9 @@ class NetworkUpdate(object):
     def interrupt(self):
         self.event_queue = []
         self.request_parent(
-            'NOTIFY', 'enqueue_event', 'OnUpdateFailure', 'artificial')
+            'NOTIFY', 'enqueue_event',
+            'OnUpdateFailure', 'artificial', '', '',
+            'ghost') # XXX
         self.state = None
         self.stop(revert=1)
 
@@ -145,7 +148,10 @@ class NetworkUpdate(object):
         return 1
 
     def start_updates(self):
-        self.enqueue_event('OnUpdateBegin')
+        self.enqueue_event(
+            'OnUpdateBegin', self.request_parent('GET', 'get_name', ''),
+            self.path, '',
+            'ghost') # XXX
         self.download(os.path.join(self.path, 'updates2.dau'))
 
     def download(self, locator, event=0):
@@ -154,7 +160,8 @@ class NetworkUpdate(object):
         if event:
             self.enqueue_event('OnUpdate.OnDownloadBegin',
                                os.path.basename(locator),
-                               self.file_number, self.num_files)
+                               self.file_number, self.num_files,
+                               'ghost') # XXX
         self.state += 1
         self.reset_timeout()
 
@@ -168,12 +175,14 @@ class NetworkUpdate(object):
         try:
             self.http.connect()
         except:
-            self.enqueue_event('OnUpdateFailure', 'connection failed')
+            self.enqueue_event('OnUpdateFailure', 'connection failed', '', '',
+                               'ghost') # XXX
             self.state = None
             self.stop(revert=1)
             return
         if self.check_timeout():
-            self.enqueue_event('OnUpdateFailure', 'timeout')
+            self.enqueue_event('OnUpdateFailure', 'timeout', '', '',
+                               'ghost') # XXX
             self.state = None
             self.stop(revert=1)
             return
@@ -185,12 +194,14 @@ class NetworkUpdate(object):
         try:
             self.response = self.http.getresponse()
         except:
-            self.enqueue_event('OnUpdateFailure', 'no HTTP response')
+            self.enqueue_event('OnUpdateFailure', 'no HTTP response', '', '',
+                               'ghost') # XXX
             self.state = None
             self.stop(revert=1)
             return
         if self.check_timeout():
-            self.enqueue_event('OnUpdateFailure', 'timeout')
+            self.enqueue_event('OnUpdateFailure', 'timeout', '', '',
+                               'ghost') # XXX
             self.state = None
             self.stop(revert=1)
             return
@@ -201,7 +212,9 @@ class NetworkUpdate(object):
         elif code == 302 and self.redirect():
             return
         elif self.state == 2: # updates2.dau
-            self.enqueue_event('OnUpdateFailure', str(code))
+            self.enqueue_event(
+                'OnUpdateFailure', str(code), 'updates2.dau', '',
+                'ghost') # XXX
             self.state = None
             return
         else:
@@ -245,14 +258,17 @@ class NetworkUpdate(object):
         data = self.response.read()
         if not data:
             if self.check_timeout():
-                self.enqueue_event('OnUpdateFailure', 'timeout')
+                self.enqueue_event('OnUpdateFailure', 'timeout', '', '',
+                                   'ghost') # XXX
                 self.state = None
                 self.stop(revert=1)
                 return
             elif data is None:
                 return
         elif data < 0:
-            self.enqueue_event('OnUpdateFailure', 'data retrieval failed')
+            self.enqueue_event(
+                'OnUpdateFailure', 'data retrieval failed', '', '',
+                'ghost') # XXX
             self.state = None
             self.stop(revert=1)
             return
@@ -283,8 +299,11 @@ class NetworkUpdate(object):
         if schedule is not None:
             self.num_files = len(schedule) - 1
             self.file_number = 0
+            update_list = ','.join([x for x, y in schedule])
             if self.num_files >= 0:
-                self.enqueue_event('OnUpdateReady', self.num_files)
+                self.enqueue_event(
+                    'OnUpdateReady', self.num_files, update_list, '',
+                    'ghost') # XXX
             self.state += 1
         return schedule
 
@@ -295,7 +314,9 @@ class NetworkUpdate(object):
             try:
                 filename, checksum, newline = line.split('\001', 2)
             except ValueError:
-                self.enqueue_event('OnUpdateFailure', 'broken updates2.dau')
+                self.enqueue_event('OnUpdateFailure', 'broken updates2.dau',
+                                   'updates2.dau', '',
+                                   'ghost') # XXX
                 self.state = None
                 return None
             if not filename:
@@ -317,6 +338,9 @@ class NetworkUpdate(object):
         return schedule
 
     def update_file(self, filename, checksum):
+        self.enqueue_event('OnUpdate.OnMD5CompareBegin',
+                           filename, '', '',
+                           'ghost') # XXX
         data = ''.join(self.buffer)
         m = hashlib.md5()
         m.update(data)
@@ -337,7 +361,9 @@ class NetworkUpdate(object):
                     os.makedirs(subdir)
                 except OSError:
                     self.enqueue_event(
-                        'OnUpdateFailure', ''.join(("can't mkdir ", subdir)))
+                        'OnUpdateFailure', ''.join(("can't mkdir ", subdir)),
+                        path, '',
+                        'ghost') # XXX
                     self.state = None
                     self.stop(revert=1)
                     return
@@ -355,14 +381,18 @@ class NetworkUpdate(object):
                     except IOError:
                         self.enqueue_event(
                             'OnUpdateFailure',
-                            ''.join(("can't write ", os.path.basename(path))))
+                            ''.join(("can't write ", os.path.basename(path))),
+                            path, '',
+                            'ghost') # XXX
                         self.state = None
                         self.stop(revert=1)
                         return
             except IOError:
                 self.enqueue_event(
                     'OnUpdateFailure',
-                    ''.join(("can't open ", os.path.basename(path))))
+                    ''.join(("can't open ", os.path.basename(path))),
+                    path, '',
+                    'ghost') # XXX
                 self.state = None
                 self.stop(revert=1)
                 return
@@ -370,7 +400,8 @@ class NetworkUpdate(object):
             event = 'OnUpdate.OnMD5CompareComplete'
         else:
             event = 'OnUpdate.OnMD5CompareFailure'
-            self.enqueue_event(event, filename, checksum, digest)
+            self.enqueue_event(event, filename, checksum, digest,
+                               'ghost') # XXX
             self.state = None
             self.stop(revert=1)
             return
@@ -391,9 +422,11 @@ class NetworkUpdate(object):
                         logging.error(e)
         update_list = ','.join(self.updated_files)
         if not update_list:
-            self.enqueue_event('OnUpdateComplete', 'none')
+            self.enqueue_event('OnUpdateComplete', 'none', '', '',
+                               'ghost') # XXX
         else:
-            self.enqueue_event('OnUpdateComplete', 'changed', update_list)
+            self.enqueue_event('OnUpdateComplete', 'changed', update_list, '',
+                               'ghost') # XXX
         self.state = None
         self.stop()
 
