@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003-2005 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -13,7 +13,7 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import optparse
+import argparse
 import urlparse
 import urllib
 import os
@@ -56,12 +56,11 @@ import ninix.menu
 from ninix.metamagic import Holon
 from ninix.metamagic import Meme
 
-USAGE = 'Usage: ninix [options]'
-parser = optparse.OptionParser(USAGE)
-parser.add_option('--sstp-port', type='int', dest='sstp_port',
-                  help='additional port for listening SSTP requests')
-parser.add_option('--debug', action='store_true', help='debug')
-parser.add_option('--logfile', type='string', help='logfile name')
+parser = argparse.ArgumentParser('ninix')
+parser.add_argument('--sstp-port', type=int, dest='sstp_port',
+                    help='additional port for listening SSTP requests')
+parser.add_argument('--debug', action='store_true', help='debug')
+parser.add_argument('--logfile', type=str, help='logfile name')
 
 # XXX: check stderr - logger's default destination and redirect it if needed
 # (See http://bugs.python.org/issue706263 for more details.)
@@ -88,8 +87,8 @@ def handleException(exception_type, value, tb):
     textview = gtk.TextView()
     textview.set_editable(False)
     left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-    width = scrn_w / 2
-    height = scrn_h / 4
+    width = scrn_w // 2
+    height = scrn_h // 4
     textview.set_size_request(width, height)
     textview.show()
     sw = gtk.ScrolledWindow()
@@ -122,11 +121,9 @@ def main():
         logging.critical('PyGtk 2.10.0 or later required')
         raise SystemExit
     # parse command line arguments
-    (options, rest) = parser.parse_args()
-    if rest:
-        parser.error('Unknown option(s)')
-    if options.logfile:
-        logger.addHandler(logging.FileHandler(options.logfile))
+    args = parser.parse_args()
+    if args.logfile is not None:
+        logger.addHandler(logging.FileHandler(args.logfile))
     # TCP 7743：伺か（未使用）(IANA Registered Port for SSTP)
     # UDP 7743：伺か（未使用）(IANA Registered Port for SSTP)
     # TCP 9801：伺か          (IANA Registered Port for SSTP)
@@ -134,13 +131,13 @@ def main():
     # TCP 9821：SSP
     # TCP 11000：伺か（廃止） (IANA Registered Port for IRISA)
     sstp_port = [9801]
-    # parse command line options
-    if options.sstp_port is not None:
-        if options.sstp_port < 1024:
+    # parse command line arguments
+    if args.sstp_port is not None:
+        if args.sstp_port < 1024 or args.sstp_port > 65535:
             logging.warning('Invalid --sstp-port number (ignored)')
         else:
-            sstp_port.append(options.sstp_port)
-    if options.debug is not None and options.debug:
+            sstp_port.append(args.sstp_port)
+    if args.debug:
         logger.setLevel(logging.DEBUG)
     home_dir = ninix.home.get_ninix_home()
     if not os.path.exists(home_dir):
@@ -148,17 +145,26 @@ def main():
             os.makedirs(home_dir)
         except:
             raise SystemExit, 'Cannot create Home directory (abort)\n'
+    lockfile_path = os.path.join(ninix.home.get_ninix_home(), '.lock')
+    if os.path.exists(lockfile_path):
+        with open(lockfile_path, 'r') as f:
+            abend = f.readline()
+        if not abend:
+            abend = None
+    else:
+        abend = None
     # aquire Inter Process Mutex (not Global Mutex)
-    with open(os.path.join(ninix.home.get_ninix_home(), '.lock'), 'w') as f:
+    with open(lockfile_path, 'w') as f:
         try:
             ninix.lock.lockfile(f)
         except:
             raise SystemExit, 'ninix-aya is already running'
         # start
         logging.info('loading...')
-        app = Application(sstp_port)
+        app = Application(f, sstp_port)
         logging.info('done.')
-        app.run()
+        app.run(abend)
+        f.truncate(0)
         try:
             ninix.lock.unlockfile(f)
         except:
@@ -254,7 +260,7 @@ class SSTPControler(object):
         self.set_sstp_flag(sender)
         self.request_parent(
             'NOTIFY', 'enqueue_script',
-            script, sender, handle, address,
+            event, script, sender, handle, address,
             show_sstp_marker, use_translator, entry_db,
             request_handler, temp_mode=True)
 
@@ -506,8 +512,13 @@ class BalloonMeme(Meme):
         desc, balloon = data
         subdir = balloon['balloon_dir'][0]
         name = desc.get('name', subdir)
+        home_dir = ninix.home.get_ninix_home()
+        thumbnail_path = os.path.join(home_dir, 'balloon',
+                                      subdir, 'thumbnail.png')
+        if not os.path.exists(thumbnail_path):
+            thumbnail_path = None
         return self.request_parent(
-            'GET', 'create_balloon_menuitem', name, self.key)
+            'GET', 'create_balloon_menuitem', name, self.key, thumbnail_path)
 
     def delete_by_myself(self):
         self.request_parent('NOTIFY', 'delete_balloon', self.key)
@@ -534,7 +545,9 @@ class Ghost(Holon):
 
 class Application(object):
 
-    def __init__(self, sstp_port=[9801, 11000]):
+    def __init__(self, lockfile, sstp_port=[9801, 11000]):
+        self.lockfile = lockfile
+        self.abend = None
         self.loaded = False
         self.confirmed = False
         self.console = Console(self)
@@ -587,9 +600,10 @@ class Application(object):
             'get_preference': self.prefs.get,
             'get_otherghostname': self.communicate.get_otherghostname,
             'rebuild_ghostdb': self.communicate.rebuild_ghostdb,
+            'notify_other': self.communicate.notify_other,
             'reset_sstp_flag': self.sstp_controler.reset_sstp_flag,
-            'send_message': self.communicate.send_message,
             'get_sstp_port': self.sstp_controler.get_sstp_port,
+            'get_prefix': self.get_sakura_prefix,
             }
         handler = handlers.get(event,
                                getattr(self, event,
@@ -598,19 +612,39 @@ class Application(object):
         if event_type == 'GET':
             return result
 
+    def set_collisionmode(self, flag, rect=False):
+        self.prefs.check_collision_button.set_active(bool(flag))
+        self.prefs.check_collision_name_button.set_active(not bool(rect))
+        self.prefs.update(commit=True) # XXX
+        self.notify_preference_changed()
+
     def do_install(self, filename):
         try:
-            filetype, target_dir = self.installer.install(
+            filetype, target_dirs = self.installer.install(
                 filename, ninix.home.get_ninix_home())
         except:
-            target_dir = None
-        if target_dir is not None:
+            target_dirs = None
+        if target_dirs:
             if filetype == 'ghost':
-                self.add_sakura(target_dir)
+                self.add_sakura(target_dirs[0])
+                ninix.sakura.ReadmeDialog().show(
+                    target_dirs[0],
+                    os.path.join(ninix.home.get_ninix_home(),
+                                 'ghost', target_dirs[0]))
+                if target_dirs[1]:
+                    self.add_balloon(target_dirs[1])
+                    ninix.sakura.ReadmeDialog().show(
+                        target_dirs[1],
+                        os.path.join(ninix.home.get_ninix_home(),
+                                     'balloon', target_dirs[1]))
             elif filetype == 'supplement':
-                self.add_sakura(target_dir) # XXX: reload
+                self.add_sakura(target_dirs) # XXX: reload
             elif filetype == 'balloon':
-                self.add_balloon(target_dir)
+                self.add_balloon(target_dirs)
+                ninix.sakura.ReadmeDialog().show(
+                    target_dirs,
+                    os.path.join(ninix.home.get_ninix_home(),
+                                 'balloon', target_dirs))
             elif filetype == 'plugin':
                 self.plugins = ninix.home.search_plugins()
             elif filetype == 'nekoninni':
@@ -620,6 +654,35 @@ class Application(object):
             elif filetype == 'kinoko':
                 self.kinoko = ninix.home.search_kinoko()
 
+    def notify_installedghostname(self, key=None):
+        installed = []
+        for value in self.ghosts.values():
+            sakura = value.instance
+            if sakura is None:
+                continue
+            installed.append(sakura.get_name(default=''))
+        if key is not None:
+            if key in self.ghosts:
+                sakura = self.ghosts[key].instance
+                sakura.notify_event('installedghostname', *installed)
+        else:
+            for sakura in self.get_working_ghost():
+                sakura.notify_event('installedghostname', *installed)
+
+    def notify_installedballoonname(self, key=None):
+        installed = []
+        for value in self.balloons.values():
+            desc, balloon = value.baseinfo
+            subdir = balloon['balloon_dir'][0]
+            installed.append(desc.get('name', subdir))
+        if key is not None:
+            if key in self.ghosts:
+                sakura = self.ghosts[key].instance
+                sakura.notify_event('installedballoonname', *installed)
+        else:
+            for sakura in self.get_working_ghost():
+                sakura.notify_event('installedballoonname', *installed)
+
     @property
     def current_sakura_instance(self):
         return self.ghosts[self.current_sakura].instance
@@ -646,13 +709,13 @@ class Application(object):
     def enqueue_event(self, event, *arglist, **argdict):
         self.current_sakura_instance.enqueue_event(event, *arglist, **argdict)
 
-    def enqueue_script(self, script, sender, handle,
+    def enqueue_script(self, event, script, sender, handle,
                        host, show_sstp_marker, use_translator,
                        db=None, request_handler=None, temp_mode=False):
         sakura = self.current_sakura_instance
         if temp_mode:
             sakura.enter_temp_mode()
-        sakura.enqueue_script(script, sender, handle,
+        sakura.enqueue_script(event, script, sender, handle,
                               host, show_sstp_marker, use_translator,
                               db, request_handler)
 
@@ -667,6 +730,9 @@ class Application(object):
                 continue
             yield sakura
 
+    def get_sakura_prefix(self):
+        return self.__menu_owner.get_prefix()
+
     def getstring(self, name):
         return self.__menu_owner.getstring(name)
 
@@ -687,7 +753,12 @@ class Application(object):
     def get_current_balloon_directory(self): ## FIXME
         return self.__menu_owner.get_current_balloon_directory()
 
-    def start_sakura_cb(self, key): ## FIXME
+    def start_sakura_cb(self, key, caller=None):
+        sakura_name = self.ghosts[key].instance.get_selfname(default='')
+        name = self.ghosts[key].instance.get_name(default='')
+        if caller is None:
+            caller = self.__menu_owner
+        caller.notify_event('OnGhostCalling', sakura_name, 'manual', name, key)
         self.start_sakura(key, init=1) # XXX
 
     def select_sakura(self, key):
@@ -736,9 +807,9 @@ class Application(object):
             menuitem.set_sensitive(key != current_key) # not working
         return self.balloon_menu
 
-    def create_balloon_menuitem(self, balloon_name, balloon_key):
+    def create_balloon_menuitem(self, balloon_name, balloon_key, thumbnail):
         return self.__menu.create_meme_menuitem(
-            balloon_name, balloon_key, self.select_balloon)
+            balloon_name, balloon_key, self.select_balloon, thumbnail)
 
     def create_balloon_menu(self):
         balloon_menuitems = OrderedDict()
@@ -749,9 +820,9 @@ class Application(object):
     def create_shell_menu(self, menuitems):
         return self.__menu.create_meme_menu(menuitems)
 
-    def create_shell_menuitem(self, shell_name, shell_key):
+    def create_shell_menuitem(self, shell_name, shell_key, thumbnail):
         return self.__menu.create_meme_menuitem(
-            shell_name, shell_key, self.select_shell)
+            shell_name, shell_key, self.select_shell, thumbnail)
 
     def create_menuitem(self, key, baseinfo):
         desc = baseinfo[0]
@@ -767,10 +838,15 @@ class Application(object):
         else:
             icon_path = None
         name = desc.get('name')
+        thumbnail_path = os.path.join(shiori_dir, 'thumbnail.png')
+        if not os.path.exists(thumbnail_path):
+            thumbnail_path = None
         start_menuitem = self.__menu.create_ghost_menuitem(
-            name, icon_path, key, self.start_sakura_cb) # XXX
+            name, icon_path, key, self.start_sakura_cb, # XXX
+            thumbnail_path)
         select_menuitem = self.__menu.create_ghost_menuitem(
-            name, icon_path, key, self.select_sakura)
+            name, icon_path, key, self.select_sakura,
+            thumbnail_path)
         menuitem = {
             'Summon': start_menuitem,
             'Change': select_menuitem,
@@ -839,7 +915,7 @@ class Application(object):
         ##    logging.info(
         ##        'GHOST({0:d}): {1}'.format(
         ##            i, name.encode('utf-8', 'ignore')))
-        self.start_sakura(self.current_sakura, init=1)
+        self.start_sakura(self.current_sakura, init=1, abend=self.abend)
 
     def find_ghost_by_dir(self, directory):
         return directory if directory in self.ghosts else None
@@ -881,7 +957,18 @@ class Application(object):
                 pass
         return None
 
-    def run(self):
+    def exit_handler(self, sig_no, frame=None):
+        self.close_all_ghosts(reason='shutdown')
+
+    def run(self, abend):
+        self.abend = abend
+        if os.name == "nt":
+            # The SIGTERM signal is not generated under Windows NT.
+            import win32api
+            win32api.SetConsoleCtrlHandler(self.exit_handler, True)
+        else:
+            import signal
+            signal.signal(signal.SIGTERM, self.exit_handler)
         self.timeout_id = glib.timeout_add(100, self.do_idle_tasks) # 100[ms]
         gtk.main()
 
@@ -908,7 +995,7 @@ class Application(object):
         sakura = self.ghosts[key].instance
         if not sakura.is_running():
             self.start_sakura(key, init=1)
-        sakura.enqueue_script('\![updatebymyself]\e', sender,
+        sakura.enqueue_script(None, '\![updatebymyself]\e', sender,
                               None, None, 0, 0, None)
 
     def select_current_sakura(self, ifghost=None):
@@ -947,9 +1034,9 @@ class Application(object):
         elif self.current_sakura == sakura.key:
             self.select_current_sakura()
 
-    def close_all_ghosts(self):
+    def close_all_ghosts(self, reason='user'):
         for sakura in self.get_working_ghost():
-            sakura.close()
+            sakura.notify_event('OnCloseAll', reason)
 
     def quit(self):
         glib.source_remove(self.timeout_id)
@@ -979,11 +1066,11 @@ class Application(object):
             key = random.choice(keys)
         self.change_sakura(sakura, key, 'automatic', event, vanished)
 
-    def select_ghost_by_name(self, sakura, name, event=1):
+    def select_ghost_by_name(self, sakura, name, event=1, vanished=0):
         key = self.find_ghost_by_name(name)
         if key is None:
             return
-        self.change_sakura(sakura, key, 'automatic', event)
+        self.change_sakura(sakura, key, 'automatic', event, vanished)
 
     def change_sakura(self, sakura, key, method, event=1, vanished=0):
         if sakura.key == key: # XXX: needs reloading?
@@ -997,8 +1084,10 @@ class Application(object):
         elif not event:
             proc()
         else:
-            name = self.ghosts[key].instance.get_selfname(default='')
-            sakura.enqueue_event('OnGhostChanging', name, method, proc=proc)
+            sakura_name = self.ghosts[key].instance.get_selfname(default='')
+            name = self.ghosts[key].instance.get_name(default='')
+            sakura.enqueue_event(
+                'OnGhostChanging', sakura_name, method, name, key, proc=proc)
 
     def stop_sakura(self, sakura, starter=None, *args):
         sakura.finalize()
@@ -1007,7 +1096,8 @@ class Application(object):
         self.set_menu_sensitive(sakura.key, True)
         self.close_ghost(sakura)
 
-    def start_sakura(self, key, prev=None, vanished=0, init=0, temp=0):
+    def start_sakura(self, key, prev=None, vanished=0, init=0, temp=0,
+                     abend=None):
         sakura = self.ghosts[key].instance
         assert sakura is not None
         if prev is not None:
@@ -1022,13 +1112,29 @@ class Application(object):
             else:
                 ghost_changed = 1
         if ghost_changed:
-            name = self.ghosts[prev].instance.get_selfname()
+            self_name = self.ghosts[prev].instance.get_selfname()
+            name = self.ghosts[prev].instance.get_name()
+            shell = self.ghosts[prev].instance.get_current_shell_name()
+            last_script = self.ghosts[prev].instance.last_script
         else:
+            self_name = None
             name = None
+            shell = None
+            last_script = None
         sakura.notify_preference_changed()
-        sakura.start(key, init, temp, vanished, ghost_changed, name)
+        sakura.start(key, init, temp, vanished, ghost_changed,
+                     self_name, name, shell, last_script, abend)
+        self.notify_installedghostname(key)
+        self.notify_installedballoonname(key)
+        sakura.notify_installedshellname()
         self.set_menu_sensitive(key, False)
 
+    def update_working(self, ghost_name):
+        self.lockfile.truncate(0)
+        self.lockfile.seek(0)
+        self.lockfile.write(ghost_name)
+        self.lockfile.flush()
+
     def notify_preference_changed(self):
         for sakura in self.get_working_ghost():
             sakura.notify_preference_changed()
@@ -1036,7 +1142,7 @@ class Application(object):
     def get_balloon_description(self, subdir): ## FIXME
         key = self.find_balloon_by_subdir(subdir)
         if key is None:
-            ##logging.warning('Balloon {0} not found.'.format(name))
+            ##logging.warning('Balloon {0} not found.'.format(subdir))
             default_balloon = self.prefs.get('default_balloon')
             key = self.find_balloon_by_subdir(default_balloon)
         if key is None:
@@ -1086,6 +1192,7 @@ class Application(object):
                 if sakura.is_running(): # stop if working
                     self.stop_sakura(sakura)
                 del self.ghosts[ghost_dir]
+        self.notify_installedghostname()
 
     def add_balloon(self, balloon_dir):
         if balloon_dir in self.balloons:
@@ -1108,7 +1215,7 @@ class Application(object):
                 del self.balloons[balloon_dir]
         self.balloon_menu = self.create_balloon_menu()
 
-    def vanish_sakura(self, sakura):
+    def vanish_sakura(self, sakura, next_ghost):
         # remove ghost
         prefix = sakura.get_prefix()
         for filename in os.listdir(prefix):
@@ -1125,7 +1232,10 @@ class Application(object):
                 except:
                     logging.error(
                         '*** REMOVE FAILED *** : {0}'.format(filename))
-        self.select_ghost(sakura, 0, vanished=1)
+        if next_ghost is not None:
+            self.select_ghost_by_name(sakura, next_ghost, vanished=1)
+        else:
+            self.select_ghost(sakura, 0, vanished=1)
         del self.ghosts[sakura.key]
 
     def select_plugin(self, item):
@@ -1216,20 +1326,18 @@ class Application(object):
 
 class Console(object):
 
-    # DnD data types
-    dnd_targets = [
-        ('text/plain', 0, 0),
-        ]
-
     def __init__(self, app):
         self.app = app
         self.window = gtk.Dialog()
         self.window.connect('delete_event', self.close)
         self.darea = gtk.DrawingArea()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
+        # DnD data types
+        dnd_targets = [('text/uri-list', 0, 0)]
         self.darea.connect('drag_data_received', self.drag_data_received)
-        self.darea.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.dnd_targets,
+        self.darea.drag_dest_set(gtk.DEST_DEFAULT_ALL, dnd_targets,
                                  gtk.gdk.ACTION_COPY)
+        self.darea.drag_dest_add_uri_targets()
         self.size = (330, 110) ## FIXME
         self.darea.set_size_request(*self.size)
         self.darea.connect('configure_event', self.configure)
@@ -1334,18 +1442,19 @@ class Console(object):
     def drag_data_received(self, widget, context, x, y, data, info, time):
         logging.info('Content-type: {0}'.format(data.type))
         logging.info('Content-length: {0:d}'.format(data.get_length()))
-        if str(data.type) == 'text/plain':
-            filelist = []
-            for line in data.data.split('\r\n'):
-                scheme, host, path, params, query, fragment = \
-                        urlparse.urlparse(line)
-                pathname = urllib.url2pathname(path)
-                if scheme == 'file' and os.path.exists(pathname):
-                    filelist.append(pathname)
+        filelist = []
+        for uri in data.get_uris():
+            scheme, host, path, params, query, fragment = \
+                urlparse.urlparse(uri)
+            pathname = urllib.url2pathname(path)
+            if scheme == 'file' and os.path.exists(pathname):
+                filelist.append(pathname)
+            elif scheme == 'http' or scheme == 'ftp':
+                filelist.append(uri)
+        if filelist:
             for filename in filelist:
                 self.app.do_install(filename)
             self.update()
-        return True
 
 
 class UsageDialog(object):
@@ -1433,7 +1542,7 @@ class UsageDialog(object):
             layout.set_text(rate)
             rate_w, rate_h = layout.get_pixel_size()
             w3 = max(rate_w, w3)
-            time = '{0}:{1:02d}'.format(*divmod(clock / 60, 60))
+            time = '{0}:{1:02d}'.format(*divmod(clock // 60, 60))
             layout.set_text(time)
             time_w, time_h = layout.get_pixel_size()
             w4 = max(time_w, w4)
